/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 1996 - 2013  SEGGER Microcontroller GmbH & Co. KG       *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

** emWin V5.22 - Graphical user interface for embedded applications **
All  Intellectual Property rights  in the Software belongs to  SEGGER.
emWin is protected by  international copyright laws.  Knowledge of the
source code may not be used to write a similar product.  This file may
only be used in accordance with the following terms:

The software has been licensed to  ARM LIMITED whose registered office
is situated at  110 Fulbourn Road,  Cambridge CB1 9NJ,  England solely
for  the  purposes  of  creating  libraries  for  ARM7, ARM9, Cortex-M
series,  and   Cortex-R4   processor-based  devices,  sublicensed  and
distributed as part of the  MDK-ARM  Professional  under the terms and
conditions  of  the   End  User  License  supplied  with  the  MDK-ARM
Professional. 
Full source code is available at: www.segger.com

We appreciate your understanding and fairness.
----------------------------------------------------------------------
File        : GUIDEMO_Bitmap.c
Purpose     : Draws bitmaps with and without compression
----------------------------------------------------------------------
*/

#include "GUIDEMO.h"

#if (SHOW_GUIDEMO_BITMAP)

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
/*********************************************************************
*
*       GUIDEMO_bm4bpp
*
**********************************************************************
*/
static const GUI_COLOR _ColorsJoerg[] = {
   0x000000, 0x111111, 0x222222, 0x333333
  ,0x444444, 0x555555, 0x666666, 0x777777
  ,0x888888, 0x999999, 0xAAAAAA, 0xBBBBBB
  ,0xCCCCCC, 0xDDDDDD, 0xEEEEEE, 0xFFFFFF
};

static const GUI_LOGPALETTE _PalJoerg = {
  16,	// Number of entries
  0, 	// No transparency
  &_ColorsJoerg[0]
};

static const unsigned char _acJoerg[] = {
  0x88, 0x88, 0x88, 0x88, 0x89, 0x88, 0x99, 0x99, 0x89, 0x99, 0x88, 0x99, 0x98, 0x88, 0x98, 0x88, 0x89, 0x88, 0x89, 0x99, 0x88, 0x89, 0x88, 0x98, 0x98, 0x89, 0x88, 0x89, 0x98, 0x88, 0x80,
  0x88, 0x88, 0x88, 0x89, 0x88, 0x88, 0x88, 0x89, 0x89, 0x99, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x89, 0x89, 0x88, 0x98, 0x88, 0x89, 0x98, 0x89, 0x88, 0x88, 0x88, 0x88, 0x80,
  0x88, 0x88, 0x88, 0x88, 0x89, 0x88, 0x88, 0x88, 0x89, 0x88, 0x88, 0x88, 0x77, 0x75, 0x68, 0x56, 0x55, 0x55, 0x68, 0x88, 0x88, 0x99, 0x98, 0x99, 0x98, 0x88, 0x88, 0x88, 0x88, 0x88, 0x80,
  0x88, 0x88, 0x88, 0x88, 0x89, 0x88, 0x88, 0x88, 0x88, 0x85, 0x44, 0x55, 0x53, 0x54, 0x35, 0x33, 0x44, 0x44, 0x46, 0x67, 0x78, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x80,
  0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x87, 0x78, 0x75, 0x43, 0x33, 0x33, 0x33, 0x34, 0x33, 0x33, 0x33, 0x44, 0x55, 0x67, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x80,
  0x98, 0x99, 0x88, 0x88, 0x88, 0x88, 0x87, 0x86, 0x63, 0x33, 0x33, 0x33, 0x33, 0x23, 0x22, 0x22, 0x23, 0x33, 0x33, 0x34, 0x56, 0x44, 0x68, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x80,
  0x88, 0x88, 0x88, 0x88, 0x88, 0x87, 0x85, 0x47, 0x54, 0x33, 0x33, 0x32, 0x22, 0x23, 0x22, 0x22, 0x22, 0x22, 0x22, 0x33, 0x33, 0x33, 0x43, 0x77, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x80,
  0x88, 0x88, 0x88, 0x88, 0x88, 0x86, 0x44, 0x43, 0x44, 0x33, 0x33, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x33, 0x23, 0x23, 0x33, 0x45, 0x77, 0x88, 0x88, 0x88, 0x88, 0x88, 0x80,
  0x88, 0x88, 0x88, 0x88, 0x87, 0x65, 0x43, 0x33, 0x33, 0x33, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x23, 0x34, 0x46, 0x76, 0x88, 0x88, 0x88, 0x88, 0x80,
  0x98, 0x88, 0x88, 0x88, 0x76, 0x54, 0x33, 0x33, 0x23, 0x32, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x12, 0x22, 0x22, 0x22, 0x22, 0x33, 0x33, 0x44, 0x48, 0x88, 0x88, 0x88, 0x88, 0x80,
  0x9A, 0x99, 0x99, 0x99, 0x75, 0x43, 0x33, 0x33, 0x22, 0x22, 0x22, 0x22, 0x22, 0x12, 0x21, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x23, 0x33, 0x33, 0x34, 0x76, 0x78, 0x88, 0x88, 0x88, 0x80,
  0x99, 0x99, 0x99, 0x97, 0x43, 0x33, 0x32, 0x32, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x21, 0x22, 0x12, 0x22, 0x12, 0x22, 0x22, 0x32, 0x22, 0x22, 0x34, 0x34, 0x88, 0x99, 0x99, 0x99, 0x90,
  0x99, 0x99, 0x99, 0x75, 0x43, 0x32, 0x22, 0x22, 0x22, 0x23, 0x23, 0x22, 0x22, 0x22, 0x21, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x23, 0x34, 0x68, 0x89, 0x89, 0x89, 0x80,
  0xAA, 0xAA, 0xA9, 0x86, 0x53, 0x22, 0x21, 0x22, 0x22, 0x23, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x24, 0x58, 0x99, 0x99, 0x99, 0x90,
  0xAA, 0xAA, 0xAA, 0x85, 0x32, 0x22, 0x21, 0x12, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x24, 0x46, 0x79, 0x99, 0x99, 0x90,
  0xAA, 0xAA, 0xA6, 0x33, 0x22, 0x22, 0x21, 0x22, 0x23, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x11, 0x22, 0x22, 0x22, 0x22, 0x22, 0x35, 0x68, 0x99, 0x99, 0x90,
  0xBB, 0xBA, 0xA4, 0x32, 0x22, 0x21, 0x21, 0x22, 0x23, 0x22, 0x22, 0x22, 0x22, 0x33, 0x33, 0x32, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x22, 0x34, 0x66, 0x99, 0x99, 0x90,
  0xBB, 0xBA, 0x83, 0x22, 0x21, 0x12, 0x22, 0x33, 0x33, 0x33, 0x33, 0x33, 0x33, 0x33, 0x33, 0x33, 0x33, 0x33, 0x23, 0x33, 0x22, 0x33, 0x32, 0x22, 0x21, 0x12, 0x22, 0x33, 0x89, 0x99, 0x90,
  0xBB, 0xBA, 0x63, 0x22, 0x21, 0x11, 0x23, 0x34, 0x44, 0x44, 0x44, 0x44, 0x44, 0x44, 0x44, 0x44, 0x44, 0x34, 0x45, 0x43, 0x33, 0x33, 0x32, 0x32, 0x21, 0x12, 0x22, 0x23, 0x69, 0x99, 0x90,
  0xBB, 0xBA, 0x32, 0x22, 0x21, 0x22, 0x34, 0x44, 0x45, 0x55, 0x55, 0x55, 0x55, 0x55, 0x44, 0x44, 0x45, 0x44, 0x54, 0x43, 0x33, 0x33, 0x33, 0x43, 0x21, 0x12, 0x22, 0x33, 0x58, 0x99, 0x90,
  0xBB, 0xB6, 0x33, 0x22, 0x22, 0x23, 0x44, 0x55, 0x55, 0x56, 0x66, 0x66, 0x76, 0x66, 0x66, 0x55, 0x66, 0x55, 0x55, 0x44, 0x44, 0x44, 0x44, 0x44, 0x32, 0x22, 0x22, 0x23, 0x36, 0x99, 0x90,
  0xBB, 0x92, 0x33, 0x22, 0x22, 0x34, 0x55, 0x55, 0x56, 0x66, 0x66, 0x66, 0x66, 0x77, 0x77, 0x66, 0x67, 0x76, 0x66, 0x66, 0x66, 0x65, 0x55, 0x55, 0x43, 0x22, 0x22, 0x22, 0x34, 0x99, 0x90,
  0xBB, 0x43, 0x32, 0x22, 0x23, 0x44, 0x55, 0x55, 0x56, 0x66, 0x66, 0x76, 0x66, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x76, 0x66, 0x66, 0x65, 0x54, 0x21, 0x12, 0x22, 0x33, 0x69, 0x90,
  0xBC, 0x33, 0x32, 0x22, 0x24, 0x45, 0x55, 0x66, 0x66, 0x66, 0x67, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x76, 0x65, 0x55, 0x42, 0x12, 0x22, 0x34, 0x79, 0x90,
  0xBB, 0x33, 0x22, 0x22, 0x34, 0x45, 0x55, 0x66, 0x66, 0x66, 0x67, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x76, 0x66, 0x55, 0x43, 0x11, 0x22, 0x33, 0x69, 0x90,
  0xBB, 0x23, 0x23, 0x22, 0x34, 0x55, 0x55, 0x66, 0x66, 0x67, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x76, 0x55, 0x43, 0x22, 0x22, 0x33, 0x49, 0x90,
  0xBB, 0x23, 0x33, 0x33, 0x34, 0x55, 0x55, 0x66, 0x67, 0x67, 0x77, 0x77, 0x77, 0x77, 0x77, 0x87, 0x77, 0x87, 0x77, 0x77, 0x77, 0x77, 0x77, 0x76, 0x65, 0x54, 0x32, 0x22, 0x33, 0x47, 0x90,
  0xBA, 0x33, 0x33, 0x33, 0x44, 0x55, 0x55, 0x66, 0x67, 0x67, 0x77, 0x77, 0x77, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x87, 0x77, 0x77, 0x77, 0x76, 0x65, 0x55, 0x32, 0x22, 0x33, 0x38, 0x90,
  0xB8, 0x33, 0x23, 0x33, 0x44, 0x55, 0x55, 0x66, 0x66, 0x77, 0x77, 0x77, 0x78, 0x88, 0x89, 0x88, 0x88, 0x88, 0x88, 0x88, 0x87, 0x77, 0x77, 0x76, 0x66, 0x55, 0x32, 0x23, 0x24, 0x38, 0x90,
  0xB9, 0x33, 0x33, 0x33, 0x45, 0x55, 0x55, 0x56, 0x66, 0x67, 0x77, 0x78, 0x88, 0x88, 0x89, 0x98, 0x99, 0x99, 0x98, 0x88, 0x87, 0x77, 0x77, 0x76, 0x66, 0x65, 0x43, 0x23, 0x33, 0x49, 0x90,
  0xB9, 0x32, 0x33, 0x34, 0x44, 0x55, 0x55, 0x56, 0x66, 0x67, 0x77, 0x88, 0x88, 0x88, 0x89, 0x88, 0x99, 0x89, 0x98, 0x88, 0x88, 0x77, 0x77, 0x76, 0x66, 0x65, 0x43, 0x33, 0x33, 0x49, 0x90,
  0xB9, 0x22, 0x33, 0x34, 0x44, 0x55, 0x55, 0x56, 0x66, 0x67, 0x77, 0x88, 0x88, 0x88, 0x99, 0x89, 0x98, 0x98, 0x88, 0x88, 0x88, 0x77, 0x76, 0x76, 0x66, 0x55, 0x43, 0x33, 0x34, 0x79, 0x90,
  0xB8, 0x32, 0x33, 0x34, 0x44, 0x55, 0x55, 0x56, 0x66, 0x67, 0x77, 0x87, 0x88, 0x88, 0x89, 0x89, 0x99, 0x98, 0x88, 0x88, 0x88, 0x77, 0x76, 0x66, 0x65, 0x55, 0x44, 0x33, 0x33, 0x89, 0x90,
  0xB5, 0x32, 0x33, 0x34, 0x44, 0x55, 0x55, 0x56, 0x66, 0x67, 0x77, 0x77, 0x88, 0x88, 0x89, 0x89, 0x99, 0x88, 0x88, 0x88, 0x78, 0x77, 0x76, 0x66, 0x66, 0x65, 0x43, 0x33, 0x33, 0x99, 0x90,
  0xB5, 0x32, 0x23, 0x34, 0x45, 0x55, 0x56, 0x66, 0x66, 0x77, 0x77, 0x77, 0x88, 0x88, 0x88, 0x88, 0x98, 0x89, 0x88, 0x88, 0x87, 0x77, 0x77, 0x66, 0x66, 0x55, 0x43, 0x33, 0x25, 0x99, 0x90,
  0xBA, 0x32, 0x23, 0x33, 0x45, 0x55, 0x66, 0x56, 0x66, 0x77, 0x77, 0x77, 0x78, 0x88, 0x89, 0x89, 0x88, 0x89, 0x88, 0x88, 0x88, 0x77, 0x77, 0x76, 0x66, 0x55, 0x43, 0x33, 0x28, 0x99, 0x90,
  0xB9, 0x23, 0x33, 0x34, 0x45, 0x55, 0x66, 0x66, 0x66, 0x77, 0x77, 0x77, 0x78, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x87, 0x77, 0x77, 0x77, 0x66, 0x65, 0x43, 0x23, 0x29, 0x99, 0x90,
  0xB9, 0x23, 0x33, 0x33, 0x55, 0x56, 0x66, 0x66, 0x66, 0x77, 0x77, 0x77, 0x88, 0x88, 0x88, 0x88, 0x88, 0x89, 0x88, 0x88, 0x78, 0x77, 0x77, 0x77, 0x76, 0x65, 0x43, 0x23, 0x39, 0x99, 0x90,
  0xBB, 0x33, 0x33, 0x34, 0x55, 0x66, 0x66, 0x66, 0x77, 0x77, 0x77, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x77, 0x77, 0x77, 0x76, 0x66, 0x53, 0x23, 0x1A, 0x99, 0x90,
  0xAA, 0x33, 0x33, 0x33, 0x55, 0x56, 0x66, 0x66, 0x66, 0x77, 0x77, 0x77, 0x88, 0x88, 0x88, 0x88, 0x88, 0x89, 0x88, 0x88, 0x88, 0x78, 0x77, 0x77, 0x76, 0x66, 0x53, 0x23, 0x1A, 0xA9, 0x90,
  0xAA, 0x23, 0x33, 0x34, 0x56, 0x66, 0x66, 0x66, 0x66, 0x66, 0x77, 0x78, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x87, 0x77, 0x77, 0x76, 0x66, 0x53, 0x23, 0x1A, 0x99, 0x90,
  0xAA, 0x73, 0x33, 0x45, 0x66, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x66, 0x78, 0x88, 0x88, 0x88, 0x88, 0x88, 0x87, 0x77, 0x66, 0x66, 0x77, 0x76, 0x66, 0x53, 0x33, 0x3A, 0x9A, 0xA0,
  0xAB, 0x84, 0x33, 0x35, 0x66, 0x56, 0x55, 0x44, 0x45, 0x43, 0x33, 0x34, 0x56, 0x67, 0x87, 0x77, 0x77, 0x88, 0x65, 0x65, 0x45, 0x55, 0x65, 0x66, 0x66, 0x66, 0x64, 0x33, 0x5A, 0x9A, 0xA0,
  0xAA, 0x33, 0x33, 0x36, 0x66, 0x65, 0x44, 0x45, 0x56, 0x55, 0x44, 0x44, 0x55, 0x66, 0x77, 0x67, 0x77, 0x77, 0x54, 0x44, 0x44, 0x44, 0x45, 0x55, 0x66, 0x66, 0x63, 0x33, 0x8A, 0x9A, 0xA0,
  0xA6, 0x63, 0x33, 0x46, 0x66, 0x65, 0x55, 0x56, 0x67, 0x77, 0x76, 0x55, 0x55, 0x66, 0x78, 0x77, 0x87, 0x76, 0x55, 0x55, 0x67, 0x77, 0x76, 0x55, 0x55, 0x66, 0x63, 0x43, 0x99, 0x9A, 0xA0,
  0xA5, 0x35, 0x33, 0x46, 0x66, 0x55, 0x55, 0x65, 0x65, 0x52, 0x34, 0x45, 0x54, 0x56, 0x78, 0x88, 0x88, 0x76, 0x66, 0x77, 0x88, 0x87, 0x77, 0x65, 0x65, 0x56, 0x63, 0x33, 0x34, 0xAA, 0xA0,
  0xB4, 0x45, 0x44, 0x46, 0x66, 0x65, 0x55, 0x43, 0x12, 0x2A, 0x12, 0x54, 0x44, 0x45, 0x68, 0x88, 0x88, 0x76, 0x66, 0x44, 0x12, 0x46, 0x77, 0x66, 0x66, 0x66, 0x63, 0x33, 0x77, 0x9A, 0xA0,
  0xB4, 0x45, 0x54, 0x46, 0x66, 0x66, 0x65, 0x32, 0x83, 0x22, 0x35, 0x75, 0x55, 0x45, 0x68, 0x88, 0x87, 0x65, 0x54, 0x51, 0x2A, 0x24, 0x24, 0x66, 0x67, 0x76, 0x63, 0x37, 0x75, 0x8A, 0xA0,
  0xB6, 0x45, 0x44, 0x56, 0x66, 0x66, 0x54, 0x55, 0x66, 0x34, 0x58, 0x56, 0x67, 0x44, 0x67, 0x88, 0x86, 0x68, 0x56, 0x83, 0x32, 0x39, 0x74, 0x46, 0x67, 0x77, 0x64, 0x57, 0x66, 0x9A, 0xA0,
  0xA7, 0x56, 0x54, 0x56, 0x66, 0x66, 0x56, 0x66, 0x66, 0x66, 0x77, 0x66, 0x66, 0x55, 0x67, 0x88, 0x87, 0x77, 0x65, 0x67, 0x44, 0x77, 0x66, 0x55, 0x67, 0x77, 0x64, 0x68, 0x76, 0x9A, 0xA0,
  0xA8, 0x56, 0x54, 0x56, 0x66, 0x66, 0x65, 0x56, 0x66, 0x77, 0x87, 0x66, 0x65, 0x55, 0x67, 0x78, 0x87, 0x67, 0x66, 0x67, 0x76, 0x77, 0x76, 0x66, 0x67, 0x76, 0x64, 0x78, 0x65, 0x9A, 0xA0,
  0xAB, 0x66, 0x43, 0x66, 0x66, 0x66, 0x66, 0x66, 0x66, 0x66, 0x66, 0x76, 0x55, 0x56, 0x67, 0x87, 0x77, 0x76, 0x77, 0x77, 0x78, 0x88, 0x77, 0x77, 0x76, 0x76, 0x64, 0x88, 0x67, 0xAA, 0xA0,
  0xAB, 0x56, 0x44, 0x56, 0x66, 0x66, 0x76, 0x66, 0x77, 0x77, 0x77, 0x66, 0x65, 0x56, 0x67, 0x77, 0x87, 0x76, 0x77, 0x87, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x65, 0x68, 0x78, 0xAA, 0xA0,
  0xAA, 0x46, 0x46, 0x56, 0x66, 0x66, 0x77, 0x77, 0x77, 0x77, 0x76, 0x77, 0x65, 0x66, 0x67, 0x88, 0x87, 0x76, 0x77, 0x88, 0x88, 0x77, 0x88, 0x77, 0x77, 0x76, 0x66, 0x68, 0x7A, 0xAA, 0xA0,
  0xBA, 0x46, 0x55, 0x56, 0x66, 0x66, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x66, 0x65, 0x67, 0x78, 0x87, 0x66, 0x77, 0x77, 0x88, 0x88, 0x88, 0x88, 0x77, 0x76, 0x67, 0x78, 0x7A, 0xAA, 0xA0,
  0xAA, 0x46, 0x63, 0x67, 0x66, 0x66, 0x67, 0x77, 0x77, 0x77, 0x77, 0x77, 0x66, 0x66, 0x67, 0x78, 0x87, 0x67, 0x77, 0x77, 0x77, 0x77, 0x77, 0x88, 0x77, 0x76, 0x76, 0x78, 0x9A, 0xAA, 0xA0,
  0xAA, 0x55, 0x65, 0x37, 0x66, 0x66, 0x67, 0x67, 0x77, 0x77, 0x77, 0x77, 0x66, 0x56, 0x66, 0x78, 0x77, 0x66, 0x77, 0x88, 0x77, 0x77, 0x78, 0x88, 0x77, 0x76, 0x86, 0x78, 0xAA, 0xAA, 0xA0,
  0xAA, 0x95, 0x76, 0x56, 0x66, 0x66, 0x66, 0x77, 0x77, 0x87, 0x77, 0x87, 0x75, 0x56, 0x67, 0x78, 0x77, 0x76, 0x88, 0x88, 0x88, 0x77, 0x78, 0x87, 0x77, 0x77, 0x67, 0x77, 0xAA, 0xAA, 0xA0,
  0xAA, 0xA5, 0x66, 0x57, 0x66, 0x66, 0x67, 0x77, 0x77, 0x87, 0x88, 0x87, 0x55, 0x56, 0x67, 0x77, 0x87, 0x66, 0x78, 0x88, 0x88, 0x88, 0x87, 0x87, 0x78, 0x76, 0x79, 0x77, 0xAA, 0xAA, 0xA0,
  0xAA, 0xA7, 0x66, 0x57, 0x66, 0x66, 0x67, 0x77, 0x77, 0x88, 0x88, 0x76, 0x56, 0x66, 0x67, 0x78, 0x87, 0x66, 0x68, 0x89, 0x88, 0x88, 0x88, 0x87, 0x77, 0x76, 0x99, 0x77, 0xAA, 0xAA, 0xA0,
  0xAA, 0xAA, 0x66, 0x56, 0x66, 0x66, 0x77, 0x77, 0x78, 0x88, 0x77, 0x65, 0x57, 0x66, 0x67, 0x8A, 0x97, 0x77, 0x77, 0x79, 0x98, 0x98, 0x88, 0x88, 0x87, 0x76, 0x88, 0x6A, 0xAA, 0xAA, 0xA0,
  0xAA, 0xAA, 0x66, 0x57, 0x66, 0x66, 0x77, 0x77, 0x78, 0x88, 0x76, 0x65, 0x57, 0x76, 0x68, 0x89, 0x88, 0x77, 0x77, 0x77, 0x89, 0x98, 0x88, 0x88, 0x87, 0x76, 0x87, 0x6A, 0xAA, 0xAA, 0xA0,
  0xAA, 0xAA, 0xA6, 0x57, 0x66, 0x66, 0x77, 0x77, 0x77, 0x77, 0x77, 0x65, 0x44, 0x46, 0x68, 0x88, 0x88, 0x77, 0x77, 0x77, 0x88, 0x98, 0x88, 0x88, 0x87, 0x76, 0x88, 0x7A, 0xAA, 0xAA, 0xA0,
  0xA9, 0xAA, 0xAA, 0xA8, 0x66, 0x66, 0x77, 0x77, 0x77, 0x77, 0x76, 0x76, 0x55, 0x44, 0x47, 0x77, 0x86, 0x55, 0x77, 0x77, 0x78, 0x88, 0x88, 0x88, 0x77, 0x76, 0x76, 0xAA, 0xAA, 0xAA, 0xA0,
  0xAA, 0xAA, 0xAA, 0xA9, 0x66, 0x66, 0x67, 0x77, 0x77, 0x77, 0x76, 0x66, 0x66, 0x54, 0x45, 0x66, 0x66, 0x66, 0x77, 0x78, 0x88, 0x88, 0x88, 0x88, 0x77, 0x66, 0x8B, 0xAA, 0xAA, 0xAA, 0xA0,
  0x9A, 0xAA, 0xA9, 0x9A, 0x77, 0x66, 0x77, 0x77, 0x77, 0x77, 0x76, 0x66, 0x66, 0x66, 0x65, 0x45, 0x56, 0x67, 0x77, 0x77, 0x87, 0x88, 0x88, 0x88, 0x77, 0x75, 0xBA, 0xAA, 0xAA, 0xAA, 0xA0,
  0xAA, 0xAA, 0xAA, 0xAA, 0x67, 0x66, 0x77, 0x77, 0x77, 0x77, 0x66, 0x66, 0x67, 0x67, 0x87, 0x77, 0x77, 0x77, 0x77, 0x77, 0x87, 0x78, 0x88, 0x88, 0x77, 0x64, 0xBA, 0xAA, 0xAA, 0xAA, 0xA0,
  0xA9, 0xAA, 0xAA, 0xAA, 0x67, 0x66, 0x67, 0x77, 0x77, 0x77, 0x56, 0x55, 0x55, 0x66, 0x77, 0x77, 0x78, 0x87, 0x67, 0x77, 0x88, 0x77, 0x88, 0x88, 0x77, 0x75, 0xAA, 0xAA, 0xAA, 0xAA, 0xA0,
  0xAA, 0xAA, 0xAA, 0xA9, 0x57, 0x66, 0x67, 0x78, 0x87, 0x76, 0x33, 0x34, 0x33, 0x45, 0x57, 0x77, 0x77, 0x77, 0x77, 0x77, 0x88, 0x88, 0x88, 0x87, 0x77, 0x79, 0xAA, 0xAA, 0xAA, 0xAA, 0xA0,
  0xAA, 0xAA, 0xAA, 0xAA, 0x76, 0x66, 0x77, 0x77, 0x89, 0x77, 0x66, 0x54, 0x44, 0x43, 0x23, 0x34, 0x34, 0x56, 0x66, 0x75, 0x68, 0x78, 0x88, 0x88, 0x77, 0x5B, 0xAA, 0xAA, 0xAA, 0xAA, 0xA0,
  0xAA, 0xAA, 0xAA, 0xAA, 0x76, 0x66, 0x75, 0x78, 0x88, 0x77, 0x76, 0x65, 0x55, 0x56, 0x65, 0x55, 0x55, 0x43, 0x34, 0x67, 0x67, 0x88, 0x88, 0x88, 0x76, 0x5A, 0xAA, 0xAA, 0xAA, 0xAA, 0xA0,
  0xA9, 0xAA, 0xAA, 0xAA, 0x76, 0x77, 0x76, 0x77, 0x78, 0x87, 0x76, 0x66, 0x65, 0x56, 0x66, 0x67, 0x66, 0x56, 0x67, 0x88, 0x88, 0x88, 0x88, 0x87, 0x76, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xA0,
  0xA9, 0xAA, 0xAA, 0xAA, 0x66, 0x67, 0x67, 0x77, 0x78, 0x87, 0x76, 0x76, 0x76, 0x66, 0x76, 0x67, 0x67, 0x77, 0x78, 0x88, 0x88, 0x88, 0x88, 0x87, 0x77, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xA0,
  0xA9, 0x9A, 0xAA, 0xAA, 0x66, 0x66, 0x77, 0x77, 0x77, 0x77, 0x76, 0x76, 0x77, 0x67, 0x66, 0x67, 0x77, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x77, 0x7B, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xA0,
  0xA9, 0xAA, 0xAA, 0xAA, 0x66, 0x56, 0x67, 0x77, 0x77, 0x77, 0x76, 0x67, 0x67, 0x77, 0x77, 0x77, 0x88, 0x88, 0x78, 0x88, 0x88, 0x88, 0x88, 0x77, 0x6A, 0xBA, 0xAA, 0xAA, 0xAA, 0xAA, 0xA0,
  0xA9, 0xAA, 0xAA, 0x74, 0x65, 0x55, 0x66, 0x66, 0x67, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x77, 0xBA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xA0,
  0xAA, 0x9A, 0xA8, 0x54, 0x65, 0x55, 0x66, 0x66, 0x67, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x78, 0x5B, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xA0,
  0x99, 0xA9, 0x97, 0x45, 0x66, 0x65, 0x66, 0x66, 0x66, 0x66, 0x77, 0x77, 0x66, 0x67, 0x77, 0x77, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x76, 0xBA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xBA, 0xA0,
  0x99, 0x99, 0x85, 0x75, 0x66, 0x66, 0x55, 0x55, 0x66, 0x66, 0x67, 0x77, 0x67, 0x77, 0x78, 0x87, 0x88, 0x88, 0x88, 0x88, 0x78, 0x77, 0x75, 0xBA, 0xAA, 0xAA, 0xAA, 0xAA, 0xBB, 0xAA, 0xA0,
  0x99, 0x99, 0x93, 0x73, 0x66, 0x66, 0x55, 0x55, 0x56, 0x66, 0x66, 0x77, 0x67, 0x78, 0x78, 0x88, 0x88, 0x88, 0x88, 0x87, 0x87, 0x77, 0x67, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xA0,
  0x99, 0x99, 0x95, 0x63, 0x65, 0x66, 0x56, 0x55, 0x55, 0x56, 0x67, 0x77, 0x67, 0x67, 0x78, 0x88, 0x88, 0x88, 0x87, 0x77, 0x77, 0x76, 0x66, 0xAB, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xA0,
  0x99, 0x99, 0x97, 0x33, 0x65, 0x55, 0x55, 0x55, 0x55, 0x55, 0x66, 0x66, 0x66, 0x77, 0x77, 0x77, 0x88, 0x87, 0x77, 0x77, 0x76, 0x66, 0x74, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xA0,
  0x99, 0x99, 0xA3, 0x32, 0x55, 0x66, 0x65, 0x55, 0x55, 0x55, 0x65, 0x66, 0x66, 0x77, 0x77, 0x77, 0x77, 0x88, 0x77, 0x77, 0x66, 0x67, 0x62, 0xA9, 0xAA, 0xBA, 0xAA, 0xBB, 0xAA, 0xAA, 0xA0,
  0x99, 0x9A, 0x91, 0x23, 0x56, 0x66, 0x66, 0x55, 0x55, 0x55, 0x55, 0x66, 0x66, 0x77, 0x77, 0x77, 0x77, 0x87, 0x77, 0x66, 0x76, 0x77, 0x62, 0x9A, 0xAA, 0xBA, 0xBA, 0xAA, 0xBA, 0xBA, 0xA0,
  0x99, 0x9A, 0xA1, 0x33, 0x56, 0x66, 0x66, 0x55, 0x55, 0x55, 0x55, 0x55, 0x66, 0x67, 0x77, 0x76, 0x76, 0x76, 0x66, 0x67, 0x77, 0x77, 0x33, 0x9A, 0xAA, 0xAA, 0xAA, 0xAA, 0xBB, 0xAB, 0xB0,
  0x99, 0x9A, 0xA2, 0x23, 0x55, 0x66, 0x66, 0x65, 0x55, 0x55, 0x55, 0x55, 0x56, 0x66, 0x66, 0x66, 0x66, 0x55, 0x56, 0x67, 0x77, 0x77, 0x23, 0x9A, 0xAA, 0xBB, 0xBB, 0xAA, 0xBA, 0xAA, 0xB0,
  0x99, 0x9A, 0xA5, 0x34, 0x55, 0x66, 0x66, 0x65, 0x66, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x66, 0x67, 0x77, 0x77, 0x77, 0x22, 0x9A, 0xAB, 0xAA, 0xBA, 0xAA, 0xBA, 0xAA, 0xA0,
  0x99, 0x9A, 0xA8, 0x34, 0x45, 0x66, 0x66, 0x56, 0x66, 0x65, 0x55, 0x55, 0x55, 0x55, 0x55, 0x56, 0x66, 0x67, 0x77, 0x77, 0x77, 0x75, 0x31, 0x8A, 0xAA, 0x9A, 0xAA, 0xBB, 0xAB, 0xBA, 0xA0,
  0x99, 0x9A, 0xA9, 0x34, 0x45, 0x66, 0x76, 0x66, 0x66, 0x65, 0x66, 0x66, 0x66, 0x65, 0x67, 0x77, 0x77, 0x77, 0x88, 0x77, 0x77, 0x75, 0x31, 0x9B, 0xBA, 0xBA, 0xAA, 0x9A, 0xBB, 0xBA, 0xB0,
  0x9A, 0x9A, 0xAA, 0x54, 0x45, 0x56, 0x66, 0x76, 0x77, 0x66, 0x66, 0x77, 0x77, 0x77, 0x67, 0x77, 0x77, 0x77, 0x78, 0x77, 0x77, 0x64, 0x31, 0xAA, 0xAA, 0xAB, 0xBB, 0xBA, 0x9A, 0xAA, 0xB0,
  0x99, 0x9A, 0xAA, 0xA3, 0x45, 0x56, 0x66, 0x77, 0x77, 0x67, 0x67, 0x77, 0x77, 0x77, 0x88, 0x88, 0x88, 0x88, 0x87, 0x77, 0x77, 0x64, 0x35, 0xAA, 0xBA, 0xBB, 0xBA, 0xAA, 0xBB, 0xAA, 0xA0,
  0xA9, 0xAA, 0xAA, 0xB8, 0x44, 0x56, 0x66, 0x77, 0x77, 0x77, 0x77, 0x77, 0x77, 0x88, 0x89, 0x88, 0x88, 0x88, 0x78, 0x77, 0x77, 0x53, 0x27, 0xAB, 0xBA, 0xBA, 0xBB, 0xBB, 0xBA, 0xAA, 0xB0
};

const GUI_BITMAP GUIDEMO_bm4bpp = {
  61,        // XSize
  92,        // YSize
  31,        // BytesPerLine
  4,         // BitsPerPixel
  _acJoerg,  // Pointer to picture data (indices)
  &_PalJoerg // Pointer to palette
};

/*********************************************************************
*
*       Bug bitmap
*
**********************************************************************
*/

static const GUI_COLOR _ColorsLadyBug[] = {
  0xFFFFFF, 0x000000, 0x0000ff, 0xFFCCCC
};

static const GUI_LOGPALETTE _PalLadyBug = {
  4,	// number of entries
  0, 	// No transparency
  &_ColorsLadyBug[0]
};

static const unsigned char _acLadyBug[] = {
  /* RLE: 056 Pixels @ 000,000*/ 56, 0x00,
  /* RLE: 007 Pixels @ 056,000*/ 7, 0x01,
  /* RLE: 152 Pixels @ 063,000*/ 152, 0x00,
  /* RLE: 009 Pixels @ 055,001*/ 9, 0x01,
  /* RLE: 150 Pixels @ 064,001*/ 150, 0x00,
  /* RLE: 010 Pixels @ 054,002*/ 10, 0x01,
  /* RLE: 149 Pixels @ 064,002*/ 149, 0x00,
  /* RLE: 012 Pixels @ 053,003*/ 12, 0x01,
  /* RLE: 148 Pixels @ 065,003*/ 148, 0x00,
  /* RLE: 012 Pixels @ 053,004*/ 12, 0x01,
  /* RLE: 147 Pixels @ 065,004*/ 147, 0x00,
  /* RLE: 013 Pixels @ 052,005*/ 13, 0x01,
  /* RLE: 147 Pixels @ 065,005*/ 147, 0x00,
  /* RLE: 013 Pixels @ 052,006*/ 13, 0x01,
  /* RLE: 146 Pixels @ 065,006*/ 146, 0x00,
  /* RLE: 014 Pixels @ 051,007*/ 14, 0x01,
  /* RLE: 146 Pixels @ 065,007*/ 146, 0x00,
  /* RLE: 014 Pixels @ 051,008*/ 14, 0x01,
  /* RLE: 146 Pixels @ 065,008*/ 146, 0x00,
  /* RLE: 014 Pixels @ 051,009*/ 14, 0x01,
  /* RLE: 145 Pixels @ 065,009*/ 145, 0x00,
  /* RLE: 014 Pixels @ 050,010*/ 14, 0x01,
  /* RLE: 146 Pixels @ 064,010*/ 146, 0x00,
  /* RLE: 014 Pixels @ 050,011*/ 14, 0x01,
  /* RLE: 146 Pixels @ 064,011*/ 146, 0x00,
  /* RLE: 014 Pixels @ 050,012*/ 14, 0x01,
  /* RLE: 146 Pixels @ 064,012*/ 146, 0x00,
  /* RLE: 013 Pixels @ 050,013*/ 13, 0x01,
  /* RLE: 146 Pixels @ 063,013*/ 146, 0x00,
  /* RLE: 014 Pixels @ 049,014*/ 14, 0x01,
  /* RLE: 146 Pixels @ 063,014*/ 146, 0x00,
  /* RLE: 013 Pixels @ 049,015*/ 13, 0x01,
  /* RLE: 147 Pixels @ 062,015*/ 147, 0x00,
  /* RLE: 013 Pixels @ 049,016*/ 13, 0x01,
  /* RLE: 147 Pixels @ 062,016*/ 147, 0x00,
  /* RLE: 012 Pixels @ 049,017*/ 12, 0x01,
  /* RLE: 148 Pixels @ 061,017*/ 148, 0x00,
  /* RLE: 012 Pixels @ 049,018*/ 12, 0x01,
  /* RLE: 148 Pixels @ 061,018*/ 148, 0x00,
  /* RLE: 012 Pixels @ 049,019*/ 12, 0x01,
  /* RLE: 147 Pixels @ 061,019*/ 147, 0x00,
  /* RLE: 012 Pixels @ 048,020*/ 12, 0x01,
  /* RLE: 148 Pixels @ 060,020*/ 148, 0x00,
  /* RLE: 012 Pixels @ 048,021*/ 12, 0x01,
  /* RLE: 148 Pixels @ 060,021*/ 148, 0x00,
  /* RLE: 011 Pixels @ 048,022*/ 11, 0x01,
  /* RLE: 149 Pixels @ 059,022*/ 149, 0x00,
  /* RLE: 011 Pixels @ 048,023*/ 11, 0x01,
  /* RLE: 149 Pixels @ 059,023*/ 149, 0x00,
  /* RLE: 011 Pixels @ 048,024*/ 11, 0x01,
  /* RLE: 149 Pixels @ 059,024*/ 149, 0x00,
  /* RLE: 010 Pixels @ 048,025*/ 10, 0x01,
  /* RLE: 150 Pixels @ 058,025*/ 150, 0x00,
  /* RLE: 010 Pixels @ 048,026*/ 10, 0x01,
  /* RLE: 150 Pixels @ 058,026*/ 150, 0x00,
  /* RLE: 010 Pixels @ 048,027*/ 10, 0x01,
  /* RLE: 150 Pixels @ 058,027*/ 150, 0x00,
  /* RLE: 009 Pixels @ 048,028*/ 9, 0x01,
  /* RLE: 152 Pixels @ 057,028*/ 152, 0x00,
  /* RLE: 008 Pixels @ 049,029*/ 8, 0x01,
  /* RLE: 152 Pixels @ 057,029*/ 152, 0x00,
  /* RLE: 008 Pixels @ 049,030*/ 8, 0x01,
  /* RLE: 152 Pixels @ 057,030*/ 152, 0x00,
  /* RLE: 008 Pixels @ 049,031*/ 8, 0x01,
  /* RLE: 152 Pixels @ 057,031*/ 152, 0x00,
  /* RLE: 008 Pixels @ 049,032*/ 8, 0x01,
  /* RLE: 152 Pixels @ 057,032*/ 152, 0x00,
  /* RLE: 008 Pixels @ 049,033*/ 8, 0x01,
  /* RLE: 152 Pixels @ 057,033*/ 152, 0x00,
  /* RLE: 008 Pixels @ 049,034*/ 8, 0x01,
  /* RLE: 153 Pixels @ 057,034*/ 153, 0x00,
  /* RLE: 007 Pixels @ 050,035*/ 7, 0x01,
  /* RLE: 153 Pixels @ 057,035*/ 153, 0x00,
  /* RLE: 007 Pixels @ 050,036*/ 7, 0x01,
  /* RLE: 153 Pixels @ 057,036*/ 153, 0x00,
  /* RLE: 007 Pixels @ 050,037*/ 7, 0x01,
  /* RLE: 153 Pixels @ 057,037*/ 153, 0x00,
  /* RLE: 007 Pixels @ 050,038*/ 7, 0x01,
  /* RLE: 154 Pixels @ 057,038*/ 154, 0x00,
  /* RLE: 006 Pixels @ 051,039*/ 6, 0x01,
  /* RLE: 154 Pixels @ 057,039*/ 154, 0x00,
  /* RLE: 007 Pixels @ 051,040*/ 7, 0x01,
  /* RLE: 154 Pixels @ 058,040*/ 154, 0x00,
  /* RLE: 006 Pixels @ 052,041*/ 6, 0x01,
  /* RLE: 154 Pixels @ 058,041*/ 154, 0x00,
  /* RLE: 007 Pixels @ 052,042*/ 7, 0x01,
  /* RLE: 154 Pixels @ 059,042*/ 154, 0x00,
  /* RLE: 006 Pixels @ 053,043*/ 6, 0x01,
  /* RLE: 154 Pixels @ 059,043*/ 154, 0x00,
  /* RLE: 007 Pixels @ 053,044*/ 7, 0x01,
  /* RLE: 049 Pixels @ 060,044*/ 49, 0x00,
  /* RLE: 010 Pixels @ 109,044*/ 10, 0x01,
  /* RLE: 095 Pixels @ 119,044*/ 95, 0x00,
  /* RLE: 006 Pixels @ 054,045*/ 6, 0x01,
  /* RLE: 046 Pixels @ 060,045*/ 46, 0x00,
  /* RLE: 014 Pixels @ 106,045*/ 14, 0x01,
  /* RLE: 095 Pixels @ 120,045*/ 95, 0x00,
  /* RLE: 006 Pixels @ 055,046*/ 6, 0x01,
  /* RLE: 043 Pixels @ 061,046*/ 43, 0x00,
  /* RLE: 017 Pixels @ 104,046*/ 17, 0x01,
  /* RLE: 094 Pixels @ 121,046*/ 94, 0x00,
  /* RLE: 007 Pixels @ 055,047*/ 7, 0x01,
  /* RLE: 040 Pixels @ 062,047*/ 40, 0x00,
  /* RLE: 020 Pixels @ 102,047*/ 20, 0x01,
  /* RLE: 054 Pixels @ 122,047*/ 54, 0x00,
  /* RLE: 019 Pixels @ 016,048*/ 19, 0x01,
  /* RLE: 021 Pixels @ 035,048*/ 21, 0x00,
  /* RLE: 006 Pixels @ 056,048*/ 6, 0x01,
  /* RLE: 004 Pixels @ 062,048*/ 4, 0x00,
  /* RLE: 006 Pixels @ 066,048*/ 6, 0x01,
  /* RLE: 029 Pixels @ 072,048*/ 29, 0x00,
  /* RLE: 022 Pixels @ 101,048*/ 22, 0x01,
  /* RLE: 049 Pixels @ 123,048*/ 49, 0x00,
  /* RLE: 027 Pixels @ 012,049*/ 27, 0x01,
  /* RLE: 018 Pixels @ 039,049*/ 18, 0x00,
  /* RLE: 006 Pixels @ 057,049*/ 6, 0x01,
  /* RLE: 001 Pixels @ 063,049*/ 1, 0x00,
  /* RLE: 009 Pixels @ 064,049*/ 9, 0x01,
  /* RLE: 026 Pixels @ 073,049*/ 26, 0x00,
  /* RLE: 024 Pixels @ 099,049*/ 24, 0x01,
  /* RLE: 046 Pixels @ 123,049*/ 46, 0x00,
  /* RLE: 032 Pixels @ 009,050*/ 32, 0x01,
  /* RLE: 017 Pixels @ 041,050*/ 17, 0x00,
  /* RLE: 016 Pixels @ 058,050*/ 16, 0x01,
  /* RLE: 024 Pixels @ 074,050*/ 24, 0x00,
  /* RLE: 026 Pixels @ 098,050*/ 26, 0x01,
  /* RLE: 043 Pixels @ 124,050*/ 43, 0x00,
  /* RLE: 036 Pixels @ 007,051*/ 36, 0x01,
  /* RLE: 015 Pixels @ 043,051*/ 15, 0x00,
  /* RLE: 017 Pixels @ 058,051*/ 17, 0x01,
  /* RLE: 022 Pixels @ 075,051*/ 22, 0x00,
  /* RLE: 027 Pixels @ 097,051*/ 27, 0x01,
  /* RLE: 040 Pixels @ 124,051*/ 40, 0x00,
  /* RLE: 041 Pixels @ 004,052*/ 41, 0x01,
  /* RLE: 014 Pixels @ 045,052*/ 14, 0x00,
  /* RLE: 016 Pixels @ 059,052*/ 16, 0x01,
  /* RLE: 021 Pixels @ 075,052*/ 21, 0x00,
  /* RLE: 016 Pixels @ 096,052*/ 16, 0x01,
  /* RLE: 005 Pixels @ 112,052*/ 5, 0x00,
  /* RLE: 012 Pixels @ 117,052*/ 12, 0x01,
  /* RLE: 034 Pixels @ 129,052*/ 34, 0x00,
  /* RLE: 044 Pixels @ 003,053*/ 44, 0x01,
  /* RLE: 010 Pixels @ 047,053*/ 10, 0x00,
  /* RLE: 018 Pixels @ 057,053*/ 18, 0x01,
  /* RLE: 020 Pixels @ 075,053*/ 20, 0x00,
  /* RLE: 016 Pixels @ 095,053*/ 16, 0x01,
  /* RLE: 007 Pixels @ 111,053*/ 7, 0x00,
  /* RLE: 013 Pixels @ 118,053*/ 13, 0x01,
  /* RLE: 031 Pixels @ 131,053*/ 31, 0x00,
  /* RLE: 046 Pixels @ 002,054*/ 46, 0x01,
  /* RLE: 008 Pixels @ 048,054*/ 8, 0x00,
  /* RLE: 020 Pixels @ 056,054*/ 20, 0x01,
  /* RLE: 018 Pixels @ 076,054*/ 18, 0x00,
  /* RLE: 015 Pixels @ 094,054*/ 15, 0x01,
  /* RLE: 008 Pixels @ 109,054*/ 8, 0x00,
  /* RLE: 016 Pixels @ 117,054*/ 16, 0x01,
  /* RLE: 028 Pixels @ 133,054*/ 28, 0x00,
  /* RLE: 049 Pixels @ 001,055*/ 49, 0x01,
  /* RLE: 005 Pixels @ 050,055*/ 5, 0x00,
  /* RLE: 021 Pixels @ 055,055*/ 21, 0x01,
  /* RLE: 003 Pixels @ 076,055*/ 3, 0x00,
  /* RLE: 030 Pixels @ 079,055*/ 30, 0x01,
  /* RLE: 006 Pixels @ 109,055*/ 6, 0x00,
  /* RLE: 018 Pixels @ 115,055*/ 18, 0x01,
  /* RLE: 027 Pixels @ 133,055*/ 27, 0x00,
  /* RLE: 028 Pixels @ 000,056*/ 28, 0x01,
  /* RLE: 012 Pixels @ 028,056*/ 12, 0x00,
  /* RLE: 011 Pixels @ 040,056*/ 11, 0x01,
  /* RLE: 003 Pixels @ 051,056*/ 3, 0x00,
  /* RLE: 054 Pixels @ 054,056*/ 54, 0x01,
  /* RLE: 006 Pixels @ 108,056*/ 6, 0x00,
  /* RLE: 020 Pixels @ 114,056*/ 20, 0x01,
  /* RLE: 026 Pixels @ 134,056*/ 26, 0x00,
  /* RLE: 025 Pixels @ 000,057*/ 25, 0x01,
  /* RLE: 018 Pixels @ 025,057*/ 18, 0x00,
  /* RLE: 009 Pixels @ 043,057*/ 9, 0x01,
  /* ABS: 002 Pixels @ 052,057*/ 0, 2, 0x00,
  /* RLE: 053 Pixels @ 054,057*/ 53, 0x01,
  /* RLE: 005 Pixels @ 107,057*/ 5, 0x00,
  /* RLE: 023 Pixels @ 112,057*/ 23, 0x01,
  /* RLE: 025 Pixels @ 135,057*/ 25, 0x00,
  /* RLE: 023 Pixels @ 000,058*/ 23, 0x01,
  /* RLE: 022 Pixels @ 023,058*/ 22, 0x00,
  /* RLE: 061 Pixels @ 045,058*/ 61, 0x01,
  /* RLE: 005 Pixels @ 106,058*/ 5, 0x00,
  /* RLE: 024 Pixels @ 111,058*/ 24, 0x01,
  /* RLE: 025 Pixels @ 135,058*/ 25, 0x00,
  /* RLE: 021 Pixels @ 000,059*/ 21, 0x01,
  /* RLE: 026 Pixels @ 021,059*/ 26, 0x00,
  /* RLE: 058 Pixels @ 047,059*/ 58, 0x01,
  /* RLE: 004 Pixels @ 105,059*/ 4, 0x00,
  /* RLE: 027 Pixels @ 109,059*/ 27, 0x01,
  /* RLE: 024 Pixels @ 136,059*/ 24, 0x00,
  /* RLE: 019 Pixels @ 000,060*/ 19, 0x01,
  /* RLE: 029 Pixels @ 019,060*/ 29, 0x00,
  /* RLE: 059 Pixels @ 048,060*/ 59, 0x01,
  /* RLE: 001 Pixels @ 107,060*/ 1, 0x00,
  /* RLE: 029 Pixels @ 108,060*/ 29, 0x01,
  /* RLE: 023 Pixels @ 137,060*/ 23, 0x00,
  /* RLE: 017 Pixels @ 000,061*/ 17, 0x01,
  /* RLE: 033 Pixels @ 017,061*/ 33, 0x00,
  /* RLE: 030 Pixels @ 050,061*/ 30, 0x01,
  /* RLE: 004 Pixels @ 080,061*/ 4, 0x03,
  /* RLE: 053 Pixels @ 084,061*/ 53, 0x01,
  /* RLE: 023 Pixels @ 137,061*/ 23, 0x00,
  /* RLE: 015 Pixels @ 000,062*/ 15, 0x01,
  /* RLE: 036 Pixels @ 015,062*/ 36, 0x00,
  /* RLE: 028 Pixels @ 051,062*/ 28, 0x01,
  /* RLE: 005 Pixels @ 079,062*/ 5, 0x03,
  /* RLE: 038 Pixels @ 084,062*/ 38, 0x01,
  /* RLE: 001 Pixels @ 122,062*/ 1, 0x00,
  /* RLE: 014 Pixels @ 123,062*/ 14, 0x01,
  /* RLE: 024 Pixels @ 137,062*/ 24, 0x00,
  /* RLE: 012 Pixels @ 001,063*/ 12, 0x01,
  /* RLE: 038 Pixels @ 013,063*/ 38, 0x00,
  /* RLE: 027 Pixels @ 051,063*/ 27, 0x01,
  /* RLE: 006 Pixels @ 078,063*/ 6, 0x03,
  /* RLE: 009 Pixels @ 084,063*/ 9, 0x01,
  /* RLE: 005 Pixels @ 093,063*/ 5, 0x02,
  /* RLE: 023 Pixels @ 098,063*/ 23, 0x01,
  /* ABS: 002 Pixels @ 121,063*/ 0, 2, 0x00,
  /* RLE: 015 Pixels @ 123,063*/ 15, 0x01,
  /* RLE: 025 Pixels @ 138,063*/ 25, 0x00,
  /* RLE: 007 Pixels @ 003,064*/ 7, 0x01,
  /* RLE: 041 Pixels @ 010,064*/ 41, 0x00,
  /* RLE: 027 Pixels @ 051,064*/ 27, 0x01,
  /* RLE: 006 Pixels @ 078,064*/ 6, 0x03,
  /* RLE: 008 Pixels @ 084,064*/ 8, 0x01,
  /* RLE: 011 Pixels @ 092,064*/ 11, 0x02,
  /* RLE: 017 Pixels @ 103,064*/ 17, 0x01,
  /* RLE: 004 Pixels @ 120,064*/ 4, 0x00,
  /* RLE: 014 Pixels @ 124,064*/ 14, 0x01,
  /* RLE: 072 Pixels @ 138,064*/ 72, 0x00,
  /* RLE: 028 Pixels @ 050,065*/ 28, 0x01,
  /* RLE: 006 Pixels @ 078,065*/ 6, 0x03,
  /* RLE: 006 Pixels @ 084,065*/ 6, 0x01,
  /* RLE: 015 Pixels @ 090,065*/ 15, 0x02,
  /* RLE: 014 Pixels @ 105,065*/ 14, 0x01,
  /* RLE: 006 Pixels @ 119,065*/ 6, 0x00,
  /* RLE: 013 Pixels @ 125,065*/ 13, 0x01,
  /* RLE: 072 Pixels @ 138,065*/ 72, 0x00,
  /* RLE: 028 Pixels @ 050,066*/ 28, 0x01,
  /* RLE: 006 Pixels @ 078,066*/ 6, 0x03,
  /* RLE: 006 Pixels @ 084,066*/ 6, 0x01,
  /* RLE: 017 Pixels @ 090,066*/ 17, 0x02,
  /* RLE: 012 Pixels @ 107,066*/ 12, 0x01,
  /* RLE: 007 Pixels @ 119,066*/ 7, 0x00,
  /* RLE: 013 Pixels @ 126,066*/ 13, 0x01,
  /* RLE: 006 Pixels @ 139,066*/ 6, 0x00,
  /* ABS: 002 Pixels @ 145,066*/ 0, 2, 0x11,
  /* RLE: 063 Pixels @ 147,066*/ 63, 0x00,
  /* RLE: 028 Pixels @ 050,067*/ 28, 0x01,
  /* RLE: 005 Pixels @ 078,067*/ 5, 0x03,
  /* RLE: 006 Pixels @ 083,067*/ 6, 0x01,
  /* RLE: 011 Pixels @ 089,067*/ 11, 0x02,
  /* ABS: 002 Pixels @ 100,067*/ 0, 2, 0x11,
  /* RLE: 006 Pixels @ 102,067*/ 6, 0x02,
  /* RLE: 010 Pixels @ 108,067*/ 10, 0x01,
  /* RLE: 009 Pixels @ 118,067*/ 9, 0x00,
  /* RLE: 013 Pixels @ 127,067*/ 13, 0x01,
  /* RLE: 004 Pixels @ 140,067*/ 4, 0x00,
  /* RLE: 004 Pixels @ 144,067*/ 4, 0x01,
  /* RLE: 062 Pixels @ 148,067*/ 62, 0x00,
  /* RLE: 038 Pixels @ 050,068*/ 38, 0x01,
  /* RLE: 010 Pixels @ 088,068*/ 10, 0x02,
  /* RLE: 006 Pixels @ 098,068*/ 6, 0x01,
  /* RLE: 006 Pixels @ 104,068*/ 6, 0x02,
  /* RLE: 007 Pixels @ 110,068*/ 7, 0x01,
  /* RLE: 012 Pixels @ 117,068*/ 12, 0x00,
  /* RLE: 012 Pixels @ 129,068*/ 12, 0x01,
  /* RLE: 003 Pixels @ 141,068*/ 3, 0x00,
  /* RLE: 004 Pixels @ 144,068*/ 4, 0x01,
  /* RLE: 062 Pixels @ 148,068*/ 62, 0x00,
  /* RLE: 038 Pixels @ 050,069*/ 38, 0x01,
  /* RLE: 009 Pixels @ 088,069*/ 9, 0x02,
  /* RLE: 008 Pixels @ 097,069*/ 8, 0x01,
  /* RLE: 006 Pixels @ 105,069*/ 6, 0x02,
  /* RLE: 007 Pixels @ 111,069*/ 7, 0x01,
  /* RLE: 013 Pixels @ 118,069*/ 13, 0x00,
  /* RLE: 011 Pixels @ 131,069*/ 11, 0x01,
  /* RLE: 001 Pixels @ 142,069*/ 1, 0x00,
  /* RLE: 006 Pixels @ 143,069*/ 6, 0x01,
  /* RLE: 061 Pixels @ 149,069*/ 61, 0x00,
  /* RLE: 037 Pixels @ 050,070*/ 37, 0x01,
  /* RLE: 009 Pixels @ 087,070*/ 9, 0x02,
  /* RLE: 010 Pixels @ 096,070*/ 10, 0x01,
  /* RLE: 006 Pixels @ 106,070*/ 6, 0x02,
  /* RLE: 006 Pixels @ 112,070*/ 6, 0x01,
  /* RLE: 010 Pixels @ 118,070*/ 10, 0x00,
  /* RLE: 021 Pixels @ 128,070*/ 21, 0x01,
  /* RLE: 061 Pixels @ 149,070*/ 61, 0x00,
  /* RLE: 037 Pixels @ 050,071*/ 37, 0x01,
  /* RLE: 008 Pixels @ 087,071*/ 8, 0x02,
  /* RLE: 012 Pixels @ 095,071*/ 12, 0x01,
  /* RLE: 006 Pixels @ 107,071*/ 6, 0x02,
  /* RLE: 006 Pixels @ 113,071*/ 6, 0x01,
  /* RLE: 007 Pixels @ 119,071*/ 7, 0x00,
  /* RLE: 023 Pixels @ 126,071*/ 23, 0x01,
  /* RLE: 061 Pixels @ 149,071*/ 61, 0x00,
  /* RLE: 037 Pixels @ 050,072*/ 37, 0x01,
  /* RLE: 008 Pixels @ 087,072*/ 8, 0x02,
  /* RLE: 012 Pixels @ 095,072*/ 12, 0x01,
  /* RLE: 007 Pixels @ 107,072*/ 7, 0x02,
  /* RLE: 006 Pixels @ 114,072*/ 6, 0x01,
  /* RLE: 005 Pixels @ 120,072*/ 5, 0x00,
  /* RLE: 025 Pixels @ 125,072*/ 25, 0x01,
  /* RLE: 060 Pixels @ 150,072*/ 60, 0x00,
  /* RLE: 036 Pixels @ 050,073*/ 36, 0x01,
  /* RLE: 008 Pixels @ 086,073*/ 8, 0x02,
  /* RLE: 014 Pixels @ 094,073*/ 14, 0x01,
  /* RLE: 007 Pixels @ 108,073*/ 7, 0x02,
  /* RLE: 006 Pixels @ 115,073*/ 6, 0x01,
  /* ABS: 002 Pixels @ 121,073*/ 0, 2, 0x00,
  /* RLE: 027 Pixels @ 123,073*/ 27, 0x01,
  /* RLE: 060 Pixels @ 150,073*/ 60, 0x00,
  /* RLE: 037 Pixels @ 050,074*/ 37, 0x01,
  /* RLE: 007 Pixels @ 087,074*/ 7, 0x02,
  /* RLE: 014 Pixels @ 094,074*/ 14, 0x01,
  /* RLE: 007 Pixels @ 108,074*/ 7, 0x02,
  /* RLE: 006 Pixels @ 115,074*/ 6, 0x01,
  /* RLE: 001 Pixels @ 121,074*/ 1, 0x00,
  /* RLE: 028 Pixels @ 122,074*/ 28, 0x01,
  /* RLE: 062 Pixels @ 150,074*/ 62, 0x00,
  /* RLE: 013 Pixels @ 052,075*/ 13, 0x01,
  /* RLE: 004 Pixels @ 065,075*/ 4, 0x03,
  /* RLE: 018 Pixels @ 069,075*/ 18, 0x01,
  /* RLE: 008 Pixels @ 087,075*/ 8, 0x02,
  /* RLE: 013 Pixels @ 095,075*/ 13, 0x01,
  /* RLE: 008 Pixels @ 108,075*/ 8, 0x02,
  /* RLE: 033 Pixels @ 116,075*/ 33, 0x01,
  /* RLE: 068 Pixels @ 149,075*/ 68, 0x00,
  /* RLE: 007 Pixels @ 057,076*/ 7, 0x01,
  /* RLE: 006 Pixels @ 064,076*/ 6, 0x03,
  /* RLE: 018 Pixels @ 070,076*/ 18, 0x01,
  /* RLE: 007 Pixels @ 088,076*/ 7, 0x02,
  /* RLE: 013 Pixels @ 095,076*/ 13, 0x01,
  /* RLE: 009 Pixels @ 108,076*/ 9, 0x02,
  /* RLE: 019 Pixels @ 117,076*/ 19, 0x01,
  /* RLE: 003 Pixels @ 136,076*/ 3, 0x00,
  /* RLE: 009 Pixels @ 139,076*/ 9, 0x01,
  /* RLE: 069 Pixels @ 148,076*/ 69, 0x00,
  /* RLE: 007 Pixels @ 057,077*/ 7, 0x01,
  /* RLE: 006 Pixels @ 064,077*/ 6, 0x03,
  /* RLE: 019 Pixels @ 070,077*/ 19, 0x01,
  /* RLE: 007 Pixels @ 089,077*/ 7, 0x02,
  /* RLE: 012 Pixels @ 096,077*/ 12, 0x01,
  /* RLE: 009 Pixels @ 108,077*/ 9, 0x02,
  /* RLE: 017 Pixels @ 117,077*/ 17, 0x01,
  /* RLE: 006 Pixels @ 134,077*/ 6, 0x00,
  /* RLE: 008 Pixels @ 140,077*/ 8, 0x01,
  /* RLE: 069 Pixels @ 148,077*/ 69, 0x00,
  /* RLE: 007 Pixels @ 057,078*/ 7, 0x01,
  /* RLE: 006 Pixels @ 064,078*/ 6, 0x03,
  /* RLE: 020 Pixels @ 070,078*/ 20, 0x01,
  /* RLE: 007 Pixels @ 090,078*/ 7, 0x02,
  /* RLE: 011 Pixels @ 097,078*/ 11, 0x01,
  /* RLE: 010 Pixels @ 108,078*/ 10, 0x02,
  /* RLE: 015 Pixels @ 118,078*/ 15, 0x01,
  /* RLE: 008 Pixels @ 133,078*/ 8, 0x00,
  /* RLE: 007 Pixels @ 141,078*/ 7, 0x01,
  /* RLE: 069 Pixels @ 148,078*/ 69, 0x00,
  /* RLE: 007 Pixels @ 057,079*/ 7, 0x01,
  /* RLE: 005 Pixels @ 064,079*/ 5, 0x03,
  /* RLE: 013 Pixels @ 069,079*/ 13, 0x01,
  /* RLE: 001 Pixels @ 082,079*/ 1, 0x02,
  /* RLE: 008 Pixels @ 083,079*/ 8, 0x01,
  /* RLE: 007 Pixels @ 091,079*/ 7, 0x02,
  /* RLE: 010 Pixels @ 098,079*/ 10, 0x01,
  /* RLE: 010 Pixels @ 108,079*/ 10, 0x02,
  /* RLE: 014 Pixels @ 118,079*/ 14, 0x01,
  /* RLE: 010 Pixels @ 132,079*/ 10, 0x00,
  /* RLE: 006 Pixels @ 142,079*/ 6, 0x01,
  /* RLE: 069 Pixels @ 148,079*/ 69, 0x00,
  /* RLE: 007 Pixels @ 057,080*/ 7, 0x01,
  /* RLE: 005 Pixels @ 064,080*/ 5, 0x03,
  /* RLE: 008 Pixels @ 069,080*/ 8, 0x01,
  /* RLE: 008 Pixels @ 077,080*/ 8, 0x02,
  /* RLE: 007 Pixels @ 085,080*/ 7, 0x01,
  /* RLE: 008 Pixels @ 092,080*/ 8, 0x02,
  /* RLE: 007 Pixels @ 100,080*/ 7, 0x01,
  /* RLE: 012 Pixels @ 107,080*/ 12, 0x02,
  /* RLE: 012 Pixels @ 119,080*/ 12, 0x01,
  /* RLE: 011 Pixels @ 131,080*/ 11, 0x00,
  /* RLE: 007 Pixels @ 142,080*/ 7, 0x01,
  /* RLE: 067 Pixels @ 149,080*/ 67, 0x00,
  /* RLE: 008 Pixels @ 056,081*/ 8, 0x01,
  /* RLE: 004 Pixels @ 064,081*/ 4, 0x03,
  /* RLE: 006 Pixels @ 068,081*/ 6, 0x01,
  /* RLE: 013 Pixels @ 074,081*/ 13, 0x02,
  /* RLE: 007 Pixels @ 087,081*/ 7, 0x01,
  /* RLE: 025 Pixels @ 094,081*/ 25, 0x02,
  /* RLE: 011 Pixels @ 119,081*/ 11, 0x01,
  /* RLE: 013 Pixels @ 130,081*/ 13, 0x00,
  /* RLE: 006 Pixels @ 143,081*/ 6, 0x01,
  /* RLE: 067 Pixels @ 149,081*/ 67, 0x00,
  /* RLE: 016 Pixels @ 056,082*/ 16, 0x01,
  /* RLE: 016 Pixels @ 072,082*/ 16, 0x02,
  /* RLE: 007 Pixels @ 088,082*/ 7, 0x01,
  /* RLE: 016 Pixels @ 095,082*/ 16, 0x02,
  /* RLE: 005 Pixels @ 111,082*/ 5, 0x01,
  /* RLE: 004 Pixels @ 116,082*/ 4, 0x02,
  /* RLE: 009 Pixels @ 120,082*/ 9, 0x01,
  /* RLE: 014 Pixels @ 129,082*/ 14, 0x00,
  /* RLE: 007 Pixels @ 143,082*/ 7, 0x01,
  /* RLE: 006 Pixels @ 150,082*/ 6, 0x00,
  /* ABS: 002 Pixels @ 156,082*/ 0, 2, 0x11,
  /* RLE: 058 Pixels @ 158,082*/ 58, 0x00,
  /* RLE: 014 Pixels @ 056,083*/ 14, 0x01,
  /* RLE: 019 Pixels @ 070,083*/ 19, 0x02,
  /* RLE: 007 Pixels @ 089,083*/ 7, 0x01,
  /* RLE: 013 Pixels @ 096,083*/ 13, 0x02,
  /* RLE: 008 Pixels @ 109,083*/ 8, 0x01,
  /* RLE: 003 Pixels @ 117,083*/ 3, 0x02,
  /* RLE: 009 Pixels @ 120,083*/ 9, 0x01,
  /* RLE: 015 Pixels @ 129,083*/ 15, 0x00,
  /* RLE: 007 Pixels @ 144,083*/ 7, 0x01,
  /* RLE: 004 Pixels @ 151,083*/ 4, 0x00,
  /* RLE: 003 Pixels @ 155,083*/ 3, 0x01,
  /* RLE: 058 Pixels @ 158,083*/ 58, 0x00,
  /* RLE: 013 Pixels @ 056,084*/ 13, 0x01,
  /* RLE: 007 Pixels @ 069,084*/ 7, 0x02,
  /* RLE: 007 Pixels @ 076,084*/ 7, 0x01,
  /* RLE: 007 Pixels @ 083,084*/ 7, 0x02,
  /* RLE: 007 Pixels @ 090,084*/ 7, 0x01,
  /* RLE: 011 Pixels @ 097,084*/ 11, 0x02,
  /* RLE: 010 Pixels @ 108,084*/ 10, 0x01,
  /* RLE: 003 Pixels @ 118,084*/ 3, 0x02,
  /* RLE: 007 Pixels @ 121,084*/ 7, 0x01,
  /* RLE: 016 Pixels @ 128,084*/ 16, 0x00,
  /* RLE: 007 Pixels @ 144,084*/ 7, 0x01,
  /* RLE: 004 Pixels @ 151,084*/ 4, 0x00,
  /* RLE: 004 Pixels @ 155,084*/ 4, 0x01,
  /* RLE: 057 Pixels @ 159,084*/ 57, 0x00,
  /* RLE: 012 Pixels @ 056,085*/ 12, 0x01,
  /* RLE: 007 Pixels @ 068,085*/ 7, 0x02,
  /* RLE: 009 Pixels @ 075,085*/ 9, 0x01,
  /* RLE: 008 Pixels @ 084,085*/ 8, 0x02,
  /* RLE: 006 Pixels @ 092,085*/ 6, 0x01,
  /* RLE: 009 Pixels @ 098,085*/ 9, 0x02,
  /* RLE: 011 Pixels @ 107,085*/ 11, 0x01,
  /* RLE: 003 Pixels @ 118,085*/ 3, 0x02,
  /* RLE: 006 Pixels @ 121,085*/ 6, 0x01,
  /* RLE: 018 Pixels @ 127,085*/ 18, 0x00,
  /* RLE: 008 Pixels @ 145,085*/ 8, 0x01,
  /* RLE: 001 Pixels @ 153,085*/ 1, 0x00,
  /* RLE: 006 Pixels @ 154,085*/ 6, 0x01,
  /* RLE: 056 Pixels @ 000,086*/ 56, 0x00,
  /* RLE: 012 Pixels @ 056,086*/ 12, 0x01,
  /* RLE: 006 Pixels @ 068,086*/ 6, 0x02,
  /* RLE: 010 Pixels @ 074,086*/ 10, 0x01,
  /* RLE: 009 Pixels @ 084,086*/ 9, 0x02,
  /* RLE: 006 Pixels @ 093,086*/ 6, 0x01,
  /* RLE: 008 Pixels @ 099,086*/ 8, 0x02,
  /* RLE: 012 Pixels @ 107,086*/ 12, 0x01,
  /* RLE: 003 Pixels @ 119,086*/ 3, 0x02,
  /* RLE: 005 Pixels @ 122,086*/ 5, 0x01,
  /* RLE: 018 Pixels @ 127,086*/ 18, 0x00,
  /* RLE: 015 Pixels @ 145,086*/ 15, 0x01,
  /* RLE: 056 Pixels @ 000,087*/ 56, 0x00,
  /* RLE: 011 Pixels @ 056,087*/ 11, 0x01,
  /* RLE: 006 Pixels @ 067,087*/ 6, 0x02,
  /* RLE: 012 Pixels @ 073,087*/ 12, 0x01,
  /* RLE: 009 Pixels @ 085,087*/ 9, 0x02,
  /* RLE: 007 Pixels @ 094,087*/ 7, 0x01,
  /* RLE: 005 Pixels @ 101,087*/ 5, 0x02,
  /* RLE: 013 Pixels @ 106,087*/ 13, 0x01,
  /* RLE: 003 Pixels @ 119,087*/ 3, 0x02,
  /* RLE: 005 Pixels @ 122,087*/ 5, 0x01,
  /* RLE: 019 Pixels @ 127,087*/ 19, 0x00,
  /* RLE: 014 Pixels @ 146,087*/ 14, 0x01,
  /* RLE: 056 Pixels @ 000,088*/ 56, 0x00,
  /* RLE: 011 Pixels @ 056,088*/ 11, 0x01,
  /* RLE: 006 Pixels @ 067,088*/ 6, 0x02,
  /* RLE: 012 Pixels @ 073,088*/ 12, 0x01,
  /* RLE: 010 Pixels @ 085,088*/ 10, 0x02,
  /* RLE: 007 Pixels @ 095,088*/ 7, 0x01,
  /* RLE: 004 Pixels @ 102,088*/ 4, 0x02,
  /* RLE: 013 Pixels @ 106,088*/ 13, 0x01,
  /* RLE: 003 Pixels @ 119,088*/ 3, 0x02,
  /* RLE: 006 Pixels @ 122,088*/ 6, 0x01,
  /* RLE: 018 Pixels @ 128,088*/ 18, 0x00,
  /* RLE: 014 Pixels @ 146,088*/ 14, 0x01,
  /* RLE: 056 Pixels @ 000,089*/ 56, 0x00,
  /* RLE: 010 Pixels @ 056,089*/ 10, 0x01,
  /* RLE: 007 Pixels @ 066,089*/ 7, 0x02,
  /* RLE: 013 Pixels @ 073,089*/ 13, 0x01,
  /* RLE: 010 Pixels @ 086,089*/ 10, 0x02,
  /* RLE: 007 Pixels @ 096,089*/ 7, 0x01,
  /* RLE: 004 Pixels @ 103,089*/ 4, 0x02,
  /* RLE: 012 Pixels @ 107,089*/ 12, 0x01,
  /* RLE: 004 Pixels @ 119,089*/ 4, 0x02,
  /* RLE: 005 Pixels @ 123,089*/ 5, 0x01,
  /* RLE: 019 Pixels @ 128,089*/ 19, 0x00,
  /* RLE: 013 Pixels @ 147,089*/ 13, 0x01,
  /* RLE: 056 Pixels @ 000,090*/ 56, 0x00,
  /* RLE: 010 Pixels @ 056,090*/ 10, 0x01,
  /* RLE: 007 Pixels @ 066,090*/ 7, 0x02,
  /* RLE: 013 Pixels @ 073,090*/ 13, 0x01,
  /* RLE: 011 Pixels @ 086,090*/ 11, 0x02,
  /* RLE: 007 Pixels @ 097,090*/ 7, 0x01,
  /* RLE: 003 Pixels @ 104,090*/ 3, 0x02,
  /* RLE: 012 Pixels @ 107,090*/ 12, 0x01,
  /* RLE: 004 Pixels @ 119,090*/ 4, 0x02,
  /* RLE: 005 Pixels @ 123,090*/ 5, 0x01,
  /* RLE: 020 Pixels @ 128,090*/ 20, 0x00,
  /* RLE: 012 Pixels @ 148,090*/ 12, 0x01,
  /* RLE: 055 Pixels @ 000,091*/ 55, 0x00,
  /* RLE: 010 Pixels @ 055,091*/ 10, 0x01,
  /* RLE: 008 Pixels @ 065,091*/ 8, 0x02,
  /* RLE: 013 Pixels @ 073,091*/ 13, 0x01,
  /* RLE: 011 Pixels @ 086,091*/ 11, 0x02,
  /* RLE: 007 Pixels @ 097,091*/ 7, 0x01,
  /* RLE: 003 Pixels @ 104,091*/ 3, 0x02,
  /* RLE: 012 Pixels @ 107,091*/ 12, 0x01,
  /* RLE: 004 Pixels @ 119,091*/ 4, 0x02,
  /* RLE: 005 Pixels @ 123,091*/ 5, 0x01,
  /* RLE: 021 Pixels @ 128,091*/ 21, 0x00,
  /* RLE: 011 Pixels @ 149,091*/ 11, 0x01,
  /* RLE: 053 Pixels @ 000,092*/ 53, 0x00,
  /* RLE: 012 Pixels @ 053,092*/ 12, 0x01,
  /* RLE: 008 Pixels @ 065,092*/ 8, 0x02,
  /* RLE: 013 Pixels @ 073,092*/ 13, 0x01,
  /* RLE: 012 Pixels @ 086,092*/ 12, 0x02,
  /* RLE: 007 Pixels @ 098,092*/ 7, 0x01,
  /* RLE: 003 Pixels @ 105,092*/ 3, 0x02,
  /* RLE: 011 Pixels @ 108,092*/ 11, 0x01,
  /* RLE: 004 Pixels @ 119,092*/ 4, 0x02,
  /* RLE: 005 Pixels @ 123,092*/ 5, 0x01,
  /* RLE: 023 Pixels @ 128,092*/ 23, 0x00,
  /* RLE: 008 Pixels @ 151,092*/ 8, 0x01,
  /* RLE: 052 Pixels @ 159,092*/ 52, 0x00,
  /* RLE: 014 Pixels @ 051,093*/ 14, 0x01,
  /* RLE: 009 Pixels @ 065,093*/ 9, 0x02,
  /* RLE: 012 Pixels @ 074,093*/ 12, 0x01,
  /* RLE: 013 Pixels @ 086,093*/ 13, 0x02,
  /* RLE: 007 Pixels @ 099,093*/ 7, 0x01,
  /* RLE: 003 Pixels @ 106,093*/ 3, 0x02,
  /* RLE: 010 Pixels @ 109,093*/ 10, 0x01,
  /* RLE: 005 Pixels @ 119,093*/ 5, 0x02,
  /* RLE: 004 Pixels @ 124,093*/ 4, 0x01,
  /* RLE: 025 Pixels @ 128,093*/ 25, 0x00,
  /* RLE: 004 Pixels @ 153,093*/ 4, 0x01,
  /* RLE: 052 Pixels @ 157,093*/ 52, 0x00,
  /* RLE: 015 Pixels @ 049,094*/ 15, 0x01,
  /* RLE: 011 Pixels @ 064,094*/ 11, 0x02,
  /* RLE: 011 Pixels @ 075,094*/ 11, 0x01,
  /* RLE: 014 Pixels @ 086,094*/ 14, 0x02,
  /* RLE: 006 Pixels @ 100,094*/ 6, 0x01,
  /* RLE: 004 Pixels @ 106,094*/ 4, 0x02,
  /* RLE: 008 Pixels @ 110,094*/ 8, 0x01,
  /* RLE: 006 Pixels @ 118,094*/ 6, 0x02,
  /* RLE: 005 Pixels @ 124,094*/ 5, 0x01,
  /* RLE: 079 Pixels @ 129,094*/ 79, 0x00,
  /* RLE: 016 Pixels @ 048,095*/ 16, 0x01,
  /* RLE: 012 Pixels @ 064,095*/ 12, 0x02,
  /* RLE: 010 Pixels @ 076,095*/ 10, 0x01,
  /* RLE: 015 Pixels @ 086,095*/ 15, 0x02,
  /* RLE: 006 Pixels @ 101,095*/ 6, 0x01,
  /* RLE: 004 Pixels @ 107,095*/ 4, 0x02,
  /* RLE: 007 Pixels @ 111,095*/ 7, 0x01,
  /* RLE: 006 Pixels @ 118,095*/ 6, 0x02,
  /* RLE: 005 Pixels @ 124,095*/ 5, 0x01,
  /* RLE: 077 Pixels @ 129,095*/ 77, 0x00,
  /* RLE: 018 Pixels @ 046,096*/ 18, 0x01,
  /* RLE: 014 Pixels @ 064,096*/ 14, 0x02,
  /* RLE: 007 Pixels @ 078,096*/ 7, 0x01,
  /* RLE: 016 Pixels @ 085,096*/ 16, 0x02,
  /* RLE: 007 Pixels @ 101,096*/ 7, 0x01,
  /* RLE: 005 Pixels @ 108,096*/ 5, 0x02,
  /* RLE: 004 Pixels @ 113,096*/ 4, 0x01,
  /* RLE: 007 Pixels @ 117,096*/ 7, 0x02,
  /* RLE: 005 Pixels @ 124,096*/ 5, 0x01,
  /* RLE: 076 Pixels @ 129,096*/ 76, 0x00,
  /* RLE: 019 Pixels @ 045,097*/ 19, 0x01,
  /* RLE: 016 Pixels @ 064,097*/ 16, 0x02,
  /* RLE: 004 Pixels @ 080,097*/ 4, 0x01,
  /* RLE: 018 Pixels @ 084,097*/ 18, 0x02,
  /* RLE: 006 Pixels @ 102,097*/ 6, 0x01,
  /* RLE: 016 Pixels @ 108,097*/ 16, 0x02,
  /* RLE: 005 Pixels @ 124,097*/ 5, 0x01,
  /* RLE: 075 Pixels @ 129,097*/ 75, 0x00,
  /* RLE: 020 Pixels @ 044,098*/ 20, 0x01,
  /* RLE: 028 Pixels @ 064,098*/ 28, 0x02,
  /* RLE: 005 Pixels @ 092,098*/ 5, 0x01,
  /* RLE: 005 Pixels @ 097,098*/ 5, 0x02,
  /* RLE: 007 Pixels @ 102,098*/ 7, 0x01,
  /* RLE: 015 Pixels @ 109,098*/ 15, 0x02,
  /* RLE: 005 Pixels @ 124,098*/ 5, 0x01,
  /* RLE: 074 Pixels @ 129,098*/ 74, 0x00,
  /* RLE: 021 Pixels @ 043,099*/ 21, 0x01,
  /* RLE: 027 Pixels @ 064,099*/ 27, 0x02,
  /* RLE: 008 Pixels @ 091,099*/ 8, 0x01,
  /* RLE: 004 Pixels @ 099,099*/ 4, 0x02,
  /* RLE: 007 Pixels @ 103,099*/ 7, 0x01,
  /* RLE: 014 Pixels @ 110,099*/ 14, 0x02,
  /* RLE: 005 Pixels @ 124,099*/ 5, 0x01,
  /* RLE: 073 Pixels @ 129,099*/ 73, 0x00,
  /* RLE: 022 Pixels @ 042,100*/ 22, 0x01,
  /* RLE: 026 Pixels @ 064,100*/ 26, 0x02,
  /* RLE: 009 Pixels @ 090,100*/ 9, 0x01,
  /* RLE: 005 Pixels @ 099,100*/ 5, 0x02,
  /* RLE: 006 Pixels @ 104,100*/ 6, 0x01,
  /* RLE: 014 Pixels @ 110,100*/ 14, 0x02,
  /* RLE: 005 Pixels @ 124,100*/ 5, 0x01,
  /* RLE: 072 Pixels @ 129,100*/ 72, 0x00,
  /* RLE: 023 Pixels @ 041,101*/ 23, 0x01,
  /* RLE: 025 Pixels @ 064,101*/ 25, 0x02,
  /* RLE: 011 Pixels @ 089,101*/ 11, 0x01,
  /* RLE: 004 Pixels @ 100,101*/ 4, 0x02,
  /* RLE: 007 Pixels @ 104,101*/ 7, 0x01,
  /* RLE: 013 Pixels @ 111,101*/ 13, 0x02,
  /* RLE: 005 Pixels @ 124,101*/ 5, 0x01,
  /* RLE: 071 Pixels @ 129,101*/ 71, 0x00,
  /* RLE: 025 Pixels @ 040,102*/ 25, 0x01,
  /* RLE: 015 Pixels @ 065,102*/ 15, 0x02,
  /* ABS: 002 Pixels @ 080,102*/ 0, 2, 0x11,
  /* RLE: 006 Pixels @ 082,102*/ 6, 0x02,
  /* RLE: 012 Pixels @ 088,102*/ 12, 0x01,
  /* RLE: 005 Pixels @ 100,102*/ 5, 0x02,
  /* RLE: 006 Pixels @ 105,102*/ 6, 0x01,
  /* RLE: 013 Pixels @ 111,102*/ 13, 0x02,
  /* RLE: 005 Pixels @ 124,102*/ 5, 0x01,
  /* RLE: 071 Pixels @ 129,102*/ 71, 0x00,
  /* RLE: 015 Pixels @ 040,103*/ 15, 0x01,
  /* RLE: 005 Pixels @ 055,103*/ 5, 0x00,
  /* RLE: 005 Pixels @ 060,103*/ 5, 0x01,
  /* RLE: 012 Pixels @ 065,103*/ 12, 0x02,
  /* RLE: 007 Pixels @ 077,103*/ 7, 0x01,
  /* RLE: 004 Pixels @ 084,103*/ 4, 0x02,
  /* RLE: 013 Pixels @ 088,103*/ 13, 0x01,
  /* RLE: 004 Pixels @ 101,103*/ 4, 0x02,
  /* RLE: 007 Pixels @ 105,103*/ 7, 0x01,
  /* RLE: 012 Pixels @ 112,103*/ 12, 0x02,
  /* RLE: 005 Pixels @ 124,103*/ 5, 0x01,
  /* RLE: 070 Pixels @ 129,103*/ 70, 0x00,
  /* RLE: 013 Pixels @ 039,104*/ 13, 0x01,
  /* RLE: 008 Pixels @ 052,104*/ 8, 0x00,
  /* RLE: 005 Pixels @ 060,104*/ 5, 0x01,
  /* RLE: 011 Pixels @ 065,104*/ 11, 0x02,
  /* RLE: 009 Pixels @ 076,104*/ 9, 0x01,
  /* RLE: 003 Pixels @ 085,104*/ 3, 0x02,
  /* RLE: 013 Pixels @ 088,104*/ 13, 0x01,
  /* RLE: 004 Pixels @ 101,104*/ 4, 0x02,
  /* RLE: 007 Pixels @ 105,104*/ 7, 0x01,
  /* RLE: 012 Pixels @ 112,104*/ 12, 0x02,
  /* RLE: 004 Pixels @ 124,104*/ 4, 0x01,
  /* RLE: 071 Pixels @ 128,104*/ 71, 0x00,
  /* RLE: 010 Pixels @ 039,105*/ 10, 0x01,
  /* RLE: 011 Pixels @ 049,105*/ 11, 0x00,
  /* RLE: 006 Pixels @ 060,105*/ 6, 0x01,
  /* RLE: 009 Pixels @ 066,105*/ 9, 0x02,
  /* RLE: 011 Pixels @ 075,105*/ 11, 0x01,
  /* ABS: 002 Pixels @ 086,105*/ 0, 2, 0x22,
  /* RLE: 013 Pixels @ 088,105*/ 13, 0x01,
  /* RLE: 005 Pixels @ 101,105*/ 5, 0x02,
  /* RLE: 007 Pixels @ 106,105*/ 7, 0x01,
  /* RLE: 010 Pixels @ 113,105*/ 10, 0x02,
  /* RLE: 005 Pixels @ 123,105*/ 5, 0x01,
  /* RLE: 071 Pixels @ 128,105*/ 71, 0x00,
  /* RLE: 009 Pixels @ 039,106*/ 9, 0x01,
  /* RLE: 013 Pixels @ 048,106*/ 13, 0x00,
  /* RLE: 005 Pixels @ 061,106*/ 5, 0x01,
  /* RLE: 009 Pixels @ 066,106*/ 9, 0x02,
  /* RLE: 011 Pixels @ 075,106*/ 11, 0x01,
  /* ABS: 002 Pixels @ 086,106*/ 0, 2, 0x22,
  /* RLE: 013 Pixels @ 088,106*/ 13, 0x01,
  /* RLE: 005 Pixels @ 101,106*/ 5, 0x02,
  /* RLE: 007 Pixels @ 106,106*/ 7, 0x01,
  /* RLE: 010 Pixels @ 113,106*/ 10, 0x02,
  /* RLE: 005 Pixels @ 123,106*/ 5, 0x01,
  /* RLE: 071 Pixels @ 128,106*/ 71, 0x00,
  /* RLE: 008 Pixels @ 039,107*/ 8, 0x01,
  /* RLE: 014 Pixels @ 047,107*/ 14, 0x00,
  /* RLE: 006 Pixels @ 061,107*/ 6, 0x01,
  /* RLE: 007 Pixels @ 067,107*/ 7, 0x02,
  /* RLE: 013 Pixels @ 074,107*/ 13, 0x01,
  /* ABS: 002 Pixels @ 087,107*/ 0, 2, 0x22,
  /* RLE: 012 Pixels @ 089,107*/ 12, 0x01,
  /* RLE: 005 Pixels @ 101,107*/ 5, 0x02,
  /* RLE: 008 Pixels @ 106,107*/ 8, 0x01,
  /* RLE: 008 Pixels @ 114,107*/ 8, 0x02,
  /* RLE: 006 Pixels @ 122,107*/ 6, 0x01,
  /* RLE: 071 Pixels @ 128,107*/ 71, 0x00,
  /* RLE: 008 Pixels @ 039,108*/ 8, 0x01,
  /* RLE: 015 Pixels @ 047,108*/ 15, 0x00,
  /* RLE: 006 Pixels @ 062,108*/ 6, 0x01,
  /* RLE: 006 Pixels @ 068,108*/ 6, 0x02,
  /* RLE: 013 Pixels @ 074,108*/ 13, 0x01,
  /* RLE: 003 Pixels @ 087,108*/ 3, 0x02,
  /* RLE: 011 Pixels @ 090,108*/ 11, 0x01,
  /* RLE: 006 Pixels @ 101,108*/ 6, 0x02,
  /* RLE: 007 Pixels @ 107,108*/ 7, 0x01,
  /* RLE: 008 Pixels @ 114,108*/ 8, 0x02,
  /* RLE: 006 Pixels @ 122,108*/ 6, 0x01,
  /* RLE: 071 Pixels @ 128,108*/ 71, 0x00,
  /* RLE: 007 Pixels @ 039,109*/ 7, 0x01,
  /* RLE: 017 Pixels @ 046,109*/ 17, 0x00,
  /* RLE: 005 Pixels @ 063,109*/ 5, 0x01,
  /* RLE: 006 Pixels @ 068,109*/ 6, 0x02,
  /* RLE: 013 Pixels @ 074,109*/ 13, 0x01,
  /* RLE: 004 Pixels @ 087,109*/ 4, 0x02,
  /* RLE: 010 Pixels @ 091,109*/ 10, 0x01,
  /* RLE: 006 Pixels @ 101,109*/ 6, 0x02,
  /* RLE: 008 Pixels @ 107,109*/ 8, 0x01,
  /* RLE: 006 Pixels @ 115,109*/ 6, 0x02,
  /* RLE: 006 Pixels @ 121,109*/ 6, 0x01,
  /* RLE: 072 Pixels @ 127,109*/ 72, 0x00,
  /* RLE: 007 Pixels @ 039,110*/ 7, 0x01,
  /* RLE: 015 Pixels @ 046,110*/ 15, 0x00,
  /* RLE: 008 Pixels @ 061,110*/ 8, 0x01,
  /* RLE: 005 Pixels @ 069,110*/ 5, 0x02,
  /* RLE: 013 Pixels @ 074,110*/ 13, 0x01,
  /* RLE: 005 Pixels @ 087,110*/ 5, 0x02,
  /* RLE: 009 Pixels @ 092,110*/ 9, 0x01,
  /* RLE: 006 Pixels @ 101,110*/ 6, 0x02,
  /* RLE: 008 Pixels @ 107,110*/ 8, 0x01,
  /* RLE: 005 Pixels @ 115,110*/ 5, 0x02,
  /* RLE: 007 Pixels @ 120,110*/ 7, 0x01,
  /* RLE: 072 Pixels @ 127,110*/ 72, 0x00,
  /* RLE: 008 Pixels @ 039,111*/ 8, 0x01,
  /* RLE: 012 Pixels @ 047,111*/ 12, 0x00,
  /* RLE: 011 Pixels @ 059,111*/ 11, 0x01,
  /* RLE: 005 Pixels @ 070,111*/ 5, 0x02,
  /* RLE: 013 Pixels @ 075,111*/ 13, 0x01,
  /* RLE: 006 Pixels @ 088,111*/ 6, 0x02,
  /* RLE: 006 Pixels @ 094,111*/ 6, 0x01,
  /* RLE: 007 Pixels @ 100,111*/ 7, 0x02,
  /* RLE: 009 Pixels @ 107,111*/ 9, 0x01,
  /* RLE: 003 Pixels @ 116,111*/ 3, 0x02,
  /* RLE: 008 Pixels @ 119,111*/ 8, 0x01,
  /* RLE: 073 Pixels @ 127,111*/ 73, 0x00,
  /* RLE: 007 Pixels @ 040,112*/ 7, 0x01,
  /* RLE: 011 Pixels @ 047,112*/ 11, 0x00,
  /* RLE: 013 Pixels @ 058,112*/ 13, 0x01,
  /* RLE: 004 Pixels @ 071,112*/ 4, 0x02,
  /* RLE: 013 Pixels @ 075,112*/ 13, 0x01,
  /* RLE: 019 Pixels @ 088,112*/ 19, 0x02,
  /* RLE: 009 Pixels @ 107,112*/ 9, 0x01,
  /* ABS: 002 Pixels @ 116,112*/ 0, 2, 0x22,
  /* RLE: 008 Pixels @ 118,112*/ 8, 0x01,
  /* RLE: 074 Pixels @ 126,112*/ 74, 0x00,
  /* RLE: 008 Pixels @ 040,113*/ 8, 0x01,
  /* RLE: 008 Pixels @ 048,113*/ 8, 0x00,
  /* RLE: 016 Pixels @ 056,113*/ 16, 0x01,
  /* RLE: 004 Pixels @ 072,113*/ 4, 0x02,
  /* RLE: 011 Pixels @ 076,113*/ 11, 0x01,
  /* RLE: 020 Pixels @ 087,113*/ 20, 0x02,
  /* RLE: 019 Pixels @ 107,113*/ 19, 0x01,
  /* RLE: 075 Pixels @ 126,113*/ 75, 0x00,
  /* RLE: 008 Pixels @ 041,114*/ 8, 0x01,
  /* RLE: 006 Pixels @ 049,114*/ 6, 0x00,
  /* RLE: 018 Pixels @ 055,114*/ 18, 0x01,
  /* RLE: 005 Pixels @ 073,114*/ 5, 0x02,
  /* RLE: 009 Pixels @ 078,114*/ 9, 0x01,
  /* RLE: 020 Pixels @ 087,114*/ 20, 0x02,
  /* RLE: 018 Pixels @ 107,114*/ 18, 0x01,
  /* RLE: 077 Pixels @ 125,114*/ 77, 0x00,
  /* RLE: 007 Pixels @ 042,115*/ 7, 0x01,
  /* RLE: 005 Pixels @ 049,115*/ 5, 0x00,
  /* RLE: 020 Pixels @ 054,115*/ 20, 0x01,
  /* RLE: 005 Pixels @ 074,115*/ 5, 0x02,
  /* RLE: 008 Pixels @ 079,115*/ 8, 0x01,
  /* RLE: 021 Pixels @ 087,115*/ 21, 0x02,
  /* RLE: 017 Pixels @ 108,115*/ 17, 0x01,
  /* RLE: 077 Pixels @ 125,115*/ 77, 0x00,
  /* RLE: 008 Pixels @ 042,116*/ 8, 0x01,
  /* RLE: 003 Pixels @ 050,116*/ 3, 0x00,
  /* RLE: 022 Pixels @ 053,116*/ 22, 0x01,
  /* RLE: 007 Pixels @ 075,116*/ 7, 0x02,
  /* RLE: 003 Pixels @ 082,116*/ 3, 0x01,
  /* RLE: 023 Pixels @ 085,116*/ 23, 0x02,
  /* RLE: 016 Pixels @ 108,116*/ 16, 0x01,
  /* RLE: 079 Pixels @ 124,116*/ 79, 0x00,
  /* RLE: 008 Pixels @ 043,117*/ 8, 0x01,
  /* RLE: 001 Pixels @ 051,117*/ 1, 0x00,
  /* RLE: 024 Pixels @ 052,117*/ 24, 0x01,
  /* RLE: 032 Pixels @ 076,117*/ 32, 0x02,
  /* RLE: 016 Pixels @ 108,117*/ 16, 0x01,
  /* RLE: 080 Pixels @ 124,117*/ 80, 0x00,
  /* RLE: 035 Pixels @ 044,118*/ 35, 0x01,
  /* RLE: 028 Pixels @ 079,118*/ 28, 0x02,
  /* RLE: 016 Pixels @ 107,118*/ 16, 0x01,
  /* RLE: 082 Pixels @ 123,118*/ 82, 0x00,
  /* RLE: 025 Pixels @ 045,119*/ 25, 0x01,
  /* RLE: 001 Pixels @ 070,119*/ 1, 0x00,
  /* RLE: 008 Pixels @ 071,119*/ 8, 0x01,
  /* RLE: 028 Pixels @ 079,119*/ 28, 0x02,
  /* RLE: 006 Pixels @ 107,119*/ 6, 0x01,
  /* RLE: 001 Pixels @ 113,119*/ 1, 0x00,
  /* RLE: 008 Pixels @ 114,119*/ 8, 0x01,
  /* RLE: 084 Pixels @ 122,119*/ 84, 0x00,
  /* RLE: 019 Pixels @ 046,120*/ 19, 0x01,
  /* RLE: 007 Pixels @ 065,120*/ 7, 0x00,
  /* RLE: 009 Pixels @ 072,120*/ 9, 0x01,
  /* RLE: 026 Pixels @ 081,120*/ 26, 0x02,
  /* RLE: 005 Pixels @ 107,120*/ 5, 0x01,
  /* RLE: 004 Pixels @ 112,120*/ 4, 0x00,
  /* RLE: 005 Pixels @ 116,120*/ 5, 0x01,
  /* RLE: 086 Pixels @ 121,120*/ 86, 0x00,
  /* RLE: 015 Pixels @ 047,121*/ 15, 0x01,
  /* RLE: 011 Pixels @ 062,121*/ 11, 0x00,
  /* RLE: 010 Pixels @ 073,121*/ 10, 0x01,
  /* RLE: 024 Pixels @ 083,121*/ 24, 0x02,
  /* RLE: 005 Pixels @ 107,121*/ 5, 0x01,
  /* RLE: 005 Pixels @ 112,121*/ 5, 0x00,
  /* RLE: 004 Pixels @ 117,121*/ 4, 0x01,
  /* RLE: 086 Pixels @ 121,121*/ 86, 0x00,
  /* RLE: 013 Pixels @ 047,122*/ 13, 0x01,
  /* RLE: 013 Pixels @ 060,122*/ 13, 0x00,
  /* RLE: 012 Pixels @ 073,122*/ 12, 0x01,
  /* RLE: 021 Pixels @ 085,122*/ 21, 0x02,
  /* RLE: 006 Pixels @ 106,122*/ 6, 0x01,
  /* RLE: 096 Pixels @ 112,122*/ 96, 0x00,
  /* RLE: 010 Pixels @ 048,123*/ 10, 0x01,
  /* RLE: 014 Pixels @ 058,123*/ 14, 0x00,
  /* RLE: 014 Pixels @ 072,123*/ 14, 0x01,
  /* RLE: 020 Pixels @ 086,123*/ 20, 0x02,
  /* RLE: 007 Pixels @ 106,123*/ 7, 0x01,
  /* RLE: 095 Pixels @ 113,123*/ 95, 0x00,
  /* RLE: 009 Pixels @ 048,124*/ 9, 0x01,
  /* RLE: 014 Pixels @ 057,124*/ 14, 0x00,
  /* RLE: 019 Pixels @ 071,124*/ 19, 0x01,
  /* RLE: 014 Pixels @ 090,124*/ 14, 0x02,
  /* RLE: 009 Pixels @ 104,124*/ 9, 0x01,
  /* RLE: 096 Pixels @ 113,124*/ 96, 0x00,
  /* RLE: 008 Pixels @ 049,125*/ 8, 0x01,
  /* RLE: 012 Pixels @ 057,125*/ 12, 0x00,
  /* RLE: 025 Pixels @ 069,125*/ 25, 0x01,
  /* RLE: 007 Pixels @ 094,125*/ 7, 0x02,
  /* RLE: 012 Pixels @ 101,125*/ 12, 0x01,
  /* RLE: 095 Pixels @ 113,125*/ 95, 0x00,
  /* RLE: 008 Pixels @ 048,126*/ 8, 0x01,
  /* RLE: 012 Pixels @ 056,126*/ 12, 0x00,
  /* RLE: 045 Pixels @ 068,126*/ 45, 0x01,
  /* RLE: 093 Pixels @ 113,126*/ 93, 0x00,
  /* RLE: 010 Pixels @ 046,127*/ 10, 0x01,
  /* RLE: 012 Pixels @ 056,127*/ 12, 0x00,
  /* RLE: 044 Pixels @ 068,127*/ 44, 0x01,
  /* RLE: 092 Pixels @ 112,127*/ 92, 0x00,
  /* RLE: 013 Pixels @ 044,128*/ 13, 0x01,
  /* RLE: 010 Pixels @ 057,128*/ 10, 0x00,
  /* RLE: 044 Pixels @ 067,128*/ 44, 0x01,
  /* RLE: 093 Pixels @ 111,128*/ 93, 0x00,
  /* RLE: 013 Pixels @ 044,129*/ 13, 0x01,
  /* RLE: 009 Pixels @ 057,129*/ 9, 0x00,
  /* RLE: 018 Pixels @ 066,129*/ 18, 0x01,
  /* RLE: 004 Pixels @ 084,129*/ 4, 0x00,
  /* RLE: 020 Pixels @ 088,129*/ 20, 0x01,
  /* RLE: 097 Pixels @ 108,129*/ 97, 0x00,
  /* RLE: 013 Pixels @ 045,130*/ 13, 0x01,
  /* RLE: 008 Pixels @ 058,130*/ 8, 0x00,
  /* RLE: 013 Pixels @ 066,130*/ 13, 0x01,
  /* RLE: 014 Pixels @ 079,130*/ 14, 0x00,
  /* RLE: 010 Pixels @ 093,130*/ 10, 0x01,
  /* RLE: 102 Pixels @ 103,130*/ 102, 0x00,
  /* RLE: 013 Pixels @ 045,131*/ 13, 0x01,
  /* RLE: 007 Pixels @ 058,131*/ 7, 0x00,
  /* RLE: 011 Pixels @ 065,131*/ 11, 0x01,
  /* RLE: 130 Pixels @ 076,131*/ 130, 0x00,
  /* RLE: 013 Pixels @ 046,132*/ 13, 0x01,
  /* RLE: 006 Pixels @ 059,132*/ 6, 0x00,
  /* RLE: 010 Pixels @ 065,132*/ 10, 0x01,
  /* RLE: 132 Pixels @ 075,132*/ 132, 0x00,
  /* RLE: 013 Pixels @ 047,133*/ 13, 0x01,
  /* RLE: 005 Pixels @ 060,133*/ 5, 0x00,
  /* RLE: 009 Pixels @ 065,133*/ 9, 0x01,
  /* RLE: 134 Pixels @ 074,133*/ 134, 0x00,
  /* RLE: 013 Pixels @ 048,134*/ 13, 0x01,
  /* RLE: 004 Pixels @ 061,134*/ 4, 0x00,
  /* RLE: 008 Pixels @ 065,134*/ 8, 0x01,
  /* RLE: 141 Pixels @ 073,134*/ 141, 0x00,
  /* RLE: 008 Pixels @ 054,135*/ 8, 0x01,
  /* RLE: 003 Pixels @ 062,135*/ 3, 0x00,
  /* RLE: 007 Pixels @ 065,135*/ 7, 0x01,
  /* RLE: 143 Pixels @ 072,135*/ 143, 0x00,
  /* RLE: 008 Pixels @ 055,136*/ 8, 0x01,
  /* ABS: 002 Pixels @ 063,136*/ 0, 2, 0x00,
  /* RLE: 007 Pixels @ 065,136*/ 7, 0x01,
  /* RLE: 144 Pixels @ 072,136*/ 144, 0x00,
  /* RLE: 008 Pixels @ 056,137*/ 8, 0x01,
  /* RLE: 001 Pixels @ 064,137*/ 1, 0x00,
  /* RLE: 007 Pixels @ 065,137*/ 7, 0x01,
  /* RLE: 145 Pixels @ 072,137*/ 145, 0x00,
  /* RLE: 007 Pixels @ 057,138*/ 7, 0x01,
  /* RLE: 001 Pixels @ 064,138*/ 1, 0x00,
  /* RLE: 008 Pixels @ 065,138*/ 8, 0x01,
  /* RLE: 144 Pixels @ 073,138*/ 144, 0x00,
  /* RLE: 008 Pixels @ 057,139*/ 8, 0x01,
  /* RLE: 001 Pixels @ 065,139*/ 1, 0x00,
  /* RLE: 007 Pixels @ 066,139*/ 7, 0x01,
  /* RLE: 145 Pixels @ 073,139*/ 145, 0x00,
  /* RLE: 007 Pixels @ 058,140*/ 7, 0x01,
  /* RLE: 001 Pixels @ 065,140*/ 1, 0x00,
  /* RLE: 008 Pixels @ 066,140*/ 8, 0x01,
  /* RLE: 144 Pixels @ 074,140*/ 144, 0x00,
  /* RLE: 007 Pixels @ 058,141*/ 7, 0x01,
  /* ABS: 002 Pixels @ 065,141*/ 0, 2, 0x00,
  /* RLE: 008 Pixels @ 067,141*/ 8, 0x01,
  /* RLE: 144 Pixels @ 075,141*/ 144, 0x00,
  /* RLE: 006 Pixels @ 059,142*/ 6, 0x01,
  /* RLE: 003 Pixels @ 065,142*/ 3, 0x00,
  /* RLE: 008 Pixels @ 068,142*/ 8, 0x01,
  /* RLE: 142 Pixels @ 076,142*/ 142, 0x00,
  /* RLE: 007 Pixels @ 058,143*/ 7, 0x01,
  /* RLE: 004 Pixels @ 065,143*/ 4, 0x00,
  /* RLE: 008 Pixels @ 069,143*/ 8, 0x01,
  /* RLE: 139 Pixels @ 077,143*/ 139, 0x00,
  /* RLE: 010 Pixels @ 056,144*/ 10, 0x01,
  /* RLE: 004 Pixels @ 066,144*/ 4, 0x00,
  /* RLE: 008 Pixels @ 070,144*/ 8, 0x01,
  /* RLE: 136 Pixels @ 078,144*/ 136, 0x00,
  /* RLE: 012 Pixels @ 054,145*/ 12, 0x01,
  /* RLE: 005 Pixels @ 066,145*/ 5, 0x00,
  /* RLE: 008 Pixels @ 071,145*/ 8, 0x01,
  /* RLE: 135 Pixels @ 079,145*/ 135, 0x00,
  /* RLE: 011 Pixels @ 054,146*/ 11, 0x01,
  /* RLE: 007 Pixels @ 065,146*/ 7, 0x00,
  /* RLE: 007 Pixels @ 072,146*/ 7, 0x01,
  /* RLE: 136 Pixels @ 079,146*/ 136, 0x00,
  /* RLE: 010 Pixels @ 055,147*/ 10, 0x01,
  /* RLE: 007 Pixels @ 065,147*/ 7, 0x00,
  /* RLE: 008 Pixels @ 072,147*/ 8, 0x01,
  /* RLE: 135 Pixels @ 080,147*/ 135, 0x00,
  /* RLE: 010 Pixels @ 055,148*/ 10, 0x01,
  /* RLE: 008 Pixels @ 065,148*/ 8, 0x00,
  /* RLE: 007 Pixels @ 073,148*/ 7, 0x01,
  /* RLE: 136 Pixels @ 080,148*/ 136, 0x00,
  /* RLE: 009 Pixels @ 056,149*/ 9, 0x01,
  /* RLE: 009 Pixels @ 065,149*/ 9, 0x00,
  /* RLE: 007 Pixels @ 074,149*/ 7, 0x01,
  /* RLE: 136 Pixels @ 081,149*/ 136, 0x00,
  /* RLE: 007 Pixels @ 057,150*/ 7, 0x01,
  /* RLE: 010 Pixels @ 064,150*/ 10, 0x00,
  /* RLE: 007 Pixels @ 074,150*/ 7, 0x01,
  /* RLE: 137 Pixels @ 081,150*/ 137, 0x00,
  /* RLE: 005 Pixels @ 058,151*/ 5, 0x01,
  /* RLE: 012 Pixels @ 063,151*/ 12, 0x00,
  /* RLE: 006 Pixels @ 075,151*/ 6, 0x01,
  /* RLE: 154 Pixels @ 081,151*/ 154, 0x00,
  /* RLE: 006 Pixels @ 075,152*/ 6, 0x01,
  /* RLE: 153 Pixels @ 081,152*/ 153, 0x00,
  /* RLE: 008 Pixels @ 074,153*/ 8, 0x01,
  /* RLE: 148 Pixels @ 082,153*/ 148, 0x00,
  /* RLE: 012 Pixels @ 070,154*/ 12, 0x01,
  /* RLE: 148 Pixels @ 082,154*/ 148, 0x00,
  /* RLE: 012 Pixels @ 070,155*/ 12, 0x01,
  /* RLE: 148 Pixels @ 082,155*/ 148, 0x00,
  /* RLE: 011 Pixels @ 070,156*/ 11, 0x01,
  /* RLE: 150 Pixels @ 081,156*/ 150, 0x00,
  /* RLE: 010 Pixels @ 071,157*/ 10, 0x01,
  /* RLE: 150 Pixels @ 081,157*/ 150, 0x00,
  /* RLE: 010 Pixels @ 071,158*/ 10, 0x01,
  /* RLE: 151 Pixels @ 081,158*/ 151, 0x00,
  /* RLE: 008 Pixels @ 072,159*/ 8, 0x01,
  /* RLE: 153 Pixels @ 080,159*/ 153, 0x00,
  /* RLE: 006 Pixels @ 073,160*/ 6, 0x01,
  /* RLE: 081 Pixels @ 079,160*/ 81, 0x00,
  0
};  /* 1927 for 25760 pixels */

static const GUI_BITMAP _bmLadyBug = {
  160,               // XSize
  161,               // YSize
  40,                // BytesPerLine
  GUI_COMPRESS_RLE4, // BitsPerPixel
  _acLadyBug,        // Pointer to picture data (indices)
  &_PalLadyBug,      // Pointer to palette
  GUI_DRAW_RLE4
};

/*********************************************************************
*
*       static code
*
**********************************************************************
*/
#define MAG_FACTOR 3

/*********************************************************************
*
*       _ShowMagnifiedBitmap
*/
static void _ShowMagnifiedBitmap(void) {
  int xCenter;
  int yCenter;
  int x0;
  int y0;

  xCenter = LCD_GetXSize() / 2;
  yCenter = LCD_GetYSize() / 2;
  x0      = xCenter - (GUIDEMO_bm4bpp.XSize >> 1) * MAG_FACTOR;
  y0      = yCenter - (GUIDEMO_bm4bpp.YSize >> 1) * MAG_FACTOR;
  GUI_DrawBitmapMag(&GUIDEMO_bm4bpp, x0, y0, MAG_FACTOR, MAG_FACTOR);
  GUIDEMO_ShowInfo("Bitmaps can be magnified...");
}



/*********************************************************************
*
*       _BitmapDemo
*/
static void _BitmapDemo(void) {
  int xSize;
  int ySize;
  int x;
  int y;

  xSize = LCD_GetXSize();
  ySize = LCD_GetYSize();
  //
  // RLE Bitmap
  //
  GUI_DrawBitmap(&_bmLadyBug, 20, 50);
  GUIDEMO_ShowInfo("RLE Compressed bitmaps");
  GUIDEMO_Wait(3000);
  GUIDEMO_DrawBk();
  //
  // 1/2/4/8 bpp bitmaps
  //
  GUI_DrawBitmap(&GUIDEMO_bm4bpp, 20, 50);
  GUIDEMO_ShowInfo("1/2/4/8 bpp bitmaps");
  GUIDEMO_Wait(3000);
  //
  // Tile display with image
  //
  for (x = 0; x < xSize / GUIDEMO_bm4bpp.XSize + 1; x++) {
    for (y = 0; y < ySize / GUIDEMO_bm4bpp.YSize + 1; y++) {
      GUI_DrawBitmap(&GUIDEMO_bm4bpp, GUIDEMO_bm4bpp.XSize * x, GUIDEMO_bm4bpp.YSize * y);
    }
  }
  GUIDEMO_Wait(2000);
  //
  // Magnified bitmap
  //
  GUIDEMO_ShowInfo("Bitmaps may also be\nmagnified and rotated");
  _ShowMagnifiedBitmap();
}

/*********************************************************************
*
*       public code
*
**********************************************************************
*/
/*********************************************************************
*
*       GUIDEMO_Bitmap
*/
void GUIDEMO_Bitmap(void) {
  GUIDEMO_ShowIntro("Bitmaps", "Showing\ndifferent bitmaps with\nand without compression");
  GUIDEMO_ShowInfoWin();
  GUIDEMO_DrawBk();
  _BitmapDemo();
  GUIDEMO_Wait(4000);
}

#else

void GUIDEMO_Bitmap_C(void);
void GUIDEMO_Bitmap_C(void) {}

#endif

/*************************** End of file ****************************/
